/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRuby.cpp,v 1.21 2002/01/24 17:05:45 ljohnson Exp $
 ***********************************************************************/

#ifdef _MSC_VER
#pragma warning (disable : 4786)
#endif

#include "FXRbCommon.h"

#include "impl.h"

#include <map>
#include <set>
#include <vector>

#ifdef __CYGWIN__
#include <io.h>		// for get_osf_handle()
#endif

#ifdef HAVE_SIGNAL_H
#include <signal.h>	// for definitions of SIGINT, etc.
#endif

// Maps C++ objects to Ruby instances
static std::map<const void*,VALUE> FXRuby_ObjMap;

// Should we catch exceptions thrown by message handlers?
FXbool FXRbCatchExceptions=FALSE;

// Returns an FXInputHandle for this Ruby file object
FXInputHandle FXRbGetInputHandle(VALUE obj) {
  int fd=FIX2INT(rb_funcall(obj,rb_intern("fileno"),0));
#ifdef WIN32
#ifdef __CYGWIN__
  return (FXInputHandle) get_osfhandle(fd);
#else
  return (FXInputHandle) _get_osfhandle(fd);
#endif
#else
  return (FXInputHandle) fd;
#endif
  }


// Register this Ruby class instance
void FXRbRegisterRubyObj(VALUE rubyObj,const void* foxObj) {
  // FXTRACE((100,"FXRbRegisterRubyObj: registering rubyObj,foxObj=%d, 0x%08x(%s)\n",rubyObj,foxObj,((FXObject*)foxObj)->getClassName()));
  FXTRACE((100,"FXRbRegisterRubyObj: registering rubyObj,foxObj=%d, 0x%08x\n",rubyObj,foxObj));
  FXRuby_ObjMap[foxObj]=rubyObj;
  FXASSERT(FXRbGetRubyObj(foxObj)==rubyObj);
  }


// Remove this mapping between a Ruby instance and a C++ object
void FXRbUnregisterRubyObj(const void* foxObj){
  // FXTRACE((100,"FXRbUnregisterRubyObj(0x%08x(%s))\n",foxObj,((FXObject*)foxObj)->getClassName()));
  FXTRACE((100,"FXRbUnregisterRubyObj(0x%08x)\n",foxObj));
  std::map<const void*,VALUE>::iterator itr=FXRuby_ObjMap.find(foxObj);
  if(itr!=FXRuby_ObjMap.end()){
    FXTRACE((100,"  unregistering Ruby instance with id %d\n",itr->second));
    FXRuby_ObjMap.erase(itr);
    DATA_PTR(itr->second)=0;
    FXASSERT(FXRuby_ObjMap.find(foxObj)==FXRuby_ObjMap.end());
    }
  }


/**
 * Return the Ruby instance associated with this C++ object (if any),
 * or Qnil if no associated Ruby instance was found.
 */
VALUE FXRbGetRubyObj(const void *foxObj) {
  std::map<const void*,VALUE>::const_iterator itr=FXRuby_ObjMap.find(foxObj);
  return (itr!=FXRuby_ObjMap.end()) ? itr->second : Qnil;
  }


// Mark this object as in use, so it won't be reclaimed by the GC
void FXRbGcMark(void *obj){
  // FXTRACE((100,"FXRbGcMark(0x%08x(%s))\n",obj,obj?((FXObject*)obj)->getClassName():""));
  FXTRACE((100,"FXRbGcMark(0x%08x)\n",obj));
  if(obj){
    VALUE value=FXRbGetRubyObj(obj);
    if(value!=Qnil){
      FXTRACE((100,"  marking Ruby object (%d) as in use\n",value));
      rb_gc_mark(value);
      }
    }
  }


/**
 * This function gets called when we need a Ruby instance that shadows
 * the corresponding C++ object. We first check our mapping to see if
 * we've already created a Ruby instance for this C++ object and, if
 * so, return that. Otherwise create a new one, register it, and return it.
 */

VALUE FXRbWrapClassInstance(VALUE klass,void* ptr){
  if(ptr){
    VALUE value=FXRbGetRubyObj(ptr);
    if(value==Qnil){
      FXTRACE((100,"FXRbWrapClassInstance: No instance found yet for ptr=0x%08x\n",ptr));
      value=Data_Wrap_Struct(klass,0,0,ptr);
      FXTRACE((100,"  Data_Wrap_Struct() returned %d\n",value));
      FXRbRegisterRubyObj(value,ptr);
      }
    else{
      FXTRACE((100,"FXRbWrapClassInstance: Found previous instance for ptr=0x%08x (%d)\n",ptr,value));
      }
    FXTRACE((100,"FXRbWrapClassInstance() returns %d\n",value));
    return value;
    }
  else{
    FXTRACE((100,"FXRbWrapClassInstance() returns Qnil\n"));
    return Qnil;
    }
  }


VALUE FXRbWrapClassInstance(VALUE module,const char* classname,void* ptr){
  FXTRACE((100,"FXRbWrapClassInstance(module=%d,classname=\"%s\",ptr=%p)\n",module,classname,ptr));
  VALUE klass=rb_const_get(module,rb_intern(classname));
  FXTRACE((100,"  rb_const_get(module,rb_intern(classname)) returned %d\n",klass));
  return FXRbWrapClassInstance(klass,ptr);
  }

//----------------------------------------------------------------------

static std::set<FXObject*> appSensitiveObjs;

void FXRbRegisterAppSensitiveObject(FXObject* obj){
  FXTRACE((100,"Registering app-sensitive object %p (%s)\n",obj,obj->getClassName()));
  appSensitiveObjs.insert(obj);
  FXASSERT(appSensitiveObjs.find(obj)!=appSensitiveObjs.end());
  }

void FXRbUnregisterAppSensitiveObject(FXObject* obj){
  FXTRACE((100,"Unregistering app-sensitive object %p (%s)\n",obj,obj->getClassName()));
  appSensitiveObjs.erase(obj);
  FXASSERT(appSensitiveObjs.find(obj)==appSensitiveObjs.end());
  }

void FXRbDestroyAppSensitiveObjects(){
  FXTRACE((100,"Destroying all app-sensitive objects...\n"));
  FXTRACE((100,"  Recording set contents in a vector...\n"));
  std::vector<FXObject*> objs;
  std::set<FXObject*>::iterator itr;
  for(itr=appSensitiveObjs.begin(); itr!=appSensitiveObjs.end(); itr++){
    objs.push_back(*itr);
    }
  appSensitiveObjs.clear();

  FXTRACE((100,"  Destroying vector objects...\n"));
  for(size_t j=0;j<objs.size();j++)
    delete objs[j];

  FXTRACE((100,"  Done.\n"));
  }

//----------------------------------------------------------------------

// Returns a Ruby array of floats
VALUE FXRbMakeArray(const FXfloat* values,FXint size){
  VALUE result=rb_ary_new();
  for(FXint i=0; i<size; i++)
    rb_ary_push(result,rb_float_new(values[i]));
  return result;
  }

// Returns a Ruby array of floats
VALUE FXRbMakeArray(const FXdouble* values,FXint size){
  VALUE result=rb_ary_new();
  for(FXint i=0; i<size; i++)
    rb_ary_push(result,rb_float_new(values[i]));
  return result;
  }

// Returns a Ruby array of integers
VALUE FXRbMakeArray(const FXint* values,FXint size){
  VALUE result=rb_ary_new();
  for(FXint i=0; i<size; i++)
    rb_ary_push(result,INT2NUM(values[i]));
  return result;
  }

//----------------------------------------------------------------------

// Massage the message data a bit
static VALUE FXRbConvertMessageData(FXObject* sender,FXSelector sel,void* ptr){
  FXTRACE((100,"FXRbConvertMessageData(%s(%p),MKUINT(%d,%s),%p)\n",sender->getClassName(),sender,SELID(sel),FXDebugTarget::messageTypeName[SELTYPE(sel)],ptr));
  FXushort type=SELTYPE(sel);
  if(type==SEL_PAINT ||
     type==SEL_LEFTBUTTONPRESS ||
     type==SEL_LEFTBUTTONRELEASE ||
     type==SEL_MIDDLEBUTTONPRESS ||
     type==SEL_MIDDLEBUTTONRELEASE ||
     type==SEL_RIGHTBUTTONPRESS ||
     type==SEL_RIGHTBUTTONRELEASE ||
     type==SEL_KEYPRESS ||
     type==SEL_KEYRELEASE ||
     type==SEL_MOTION ||
     type==SEL_BEGINDRAG ||
     type==SEL_ENDDRAG ||
     type==SEL_DRAGGED ||
     type==SEL_ENTER ||
     type==SEL_LEAVE ||
     type==SEL_MAP ||
     type==SEL_UNMAP ||
     type==SEL_CONFIGURE ||
     type==SEL_FOCUSIN ||
     type==SEL_FOCUSOUT ||
     type==SEL_DND_ENTER ||
     type==SEL_DND_LEAVE ||
     type==SEL_DND_MOTION ||
     type==SEL_DND_DROP ||
     type==SEL_DND_REQUEST ||
     type==SEL_SELECTION_LOST ||
     type==SEL_SELECTION_GAINED ||
     type==SEL_SELECTION_REQUEST ||
     type==SEL_CLIPBOARD_LOST ||
     type==SEL_CLIPBOARD_GAINED ||
     type==SEL_CLIPBOARD_REQUEST ||
     type==SEL_UNGRABBED) {
    return to_ruby((FXEvent*)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXArrowButton))){
    if(type==SEL_CLICKED||type==SEL_COMMAND) return to_ruby((FXuint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXButton))){
    if(type==SEL_CLICKED ||
       type==SEL_DOUBLECLICKED ||
       type==SEL_TRIPLECLICKED ||
       type==SEL_COMMAND) return to_ruby((FXuint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXCheckButton))){
    if(type==SEL_COMMAND) return to_ruby((FXuchar)(long)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXColorSelector))){
    if(type==SEL_CHANGED || type==SEL_COMMAND) return to_ruby((FXColor)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXColorWell))){
    if(type==SEL_CHANGED ||
       type==SEL_COMMAND ||
       type==SEL_CLICKED ||
       type==SEL_DOUBLECLICKED ||
       type==SEL_TRIPLECLICKED) return to_ruby((FXColor)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXComboBox))){
    if(type==SEL_CHANGED || type==SEL_COMMAND) return to_ruby((const FXchar*)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXDial))){
    if(type==SEL_CHANGED || type==SEL_COMMAND) return to_ruby((FXint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXDirBox))){
    if(type==SEL_CHANGED || type==SEL_COMMAND) return to_ruby((const FXchar*) ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXFileList))){
    if (type==SEL_CHANGED ||
        type==SEL_CLICKED ||
        type==SEL_DOUBLECLICKED ||
        type==SEL_TRIPLECLICKED ||
        type==SEL_COMMAND) return to_ruby((FXint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXGLViewer))){
    if(type==SEL_CHANGED ||
       type==SEL_CLICKED ||
       type==SEL_DOUBLECLICKED ||
       type==SEL_TRIPLECLICKED) return FXRbWrapClassInstance(cFX_GLObject,ptr);
    else if(type==SEL_COMMAND){
      if(SELID(sel)==FXWindow::ID_QUERY_MENU)
        return to_ruby((FXEvent*)ptr);
      else
        return FXRbWrapClassInstance(cFX_GLObject,ptr);
      }
    }
  else if(sender->isMemberOf(FXMETACLASS(FXHeader))){
    if(type==SEL_COMMAND || type==SEL_CHANGED) return to_ruby((FXint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXIconList))){
    if(type==SEL_CHANGED ||
       type==SEL_CLICKED ||
       type==SEL_DOUBLECLICKED ||
       type==SEL_TRIPLECLICKED ||
       type==SEL_COMMAND) return to_ruby((FXint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXList))){
    if(type==SEL_CHANGED ||
       type==SEL_CLICKED ||
       type==SEL_DOUBLECLICKED ||
       type==SEL_TRIPLECLICKED ||
       type==SEL_SELECTED ||
       type==SEL_DESELECTED ||
       type==SEL_REPLACED ||
       type==SEL_INSERTED ||
       type==SEL_DELETED ||
       type==SEL_COMMAND) return to_ruby((FXint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXMDIChild))){
    if(type==SEL_SELECTED || type==SEL_DESELECTED) return FXRbWrapClassInstance(cFX_MDIChild,ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXMDIClient))){
    if(type==SEL_CHANGED) return FXRbWrapClassInstance(cFX_MDIChild,ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXRadioButton))){
    if(type==SEL_COMMAND) return to_ruby((FXuchar)(long)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXRecentFiles))){
    if(type==SEL_COMMAND) return to_ruby((const FXchar*)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXScrollbar))){
    if(type==SEL_CHANGED || type==SEL_COMMAND) return to_ruby((FXint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXShutter))){
    if(type==SEL_COMMAND) return to_ruby((FXint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXSlider))){
    if(type==SEL_CHANGED || type==SEL_COMMAND)
      return to_ruby((FXint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXTabBar))){
    if(type==SEL_COMMAND) return to_ruby((FXint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXTabBook))){
    if(type==SEL_COMMAND) return to_ruby((FXint)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXTable))){
    if(type==SEL_CLICKED ||
       type==SEL_DOUBLECLICKED ||
       type==SEL_TRIPLECLICKED ||
       type==SEL_CHANGED ||
       type==SEL_COMMAND) return to_ruby((FXTablePos*)ptr);
    else if(type==SEL_SELECTED ||
            type == SEL_DESELECTED ||
            type == SEL_INSERTED ||
            type == SEL_DELETED) return to_ruby((FXTableRange*)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXText))){
    if     (type==SEL_CHANGED) return to_ruby((FXint)ptr);
    else if(type==SEL_SELECTED ||
            type == SEL_DESELECTED ||
            type == SEL_INSERTED ||
            type == SEL_DELETED) {
      FXint* what=(FXint*)ptr;
      VALUE ary=rb_ary_new();
      rb_ary_push(ary,to_ruby(what[0])); // start position of text
      rb_ary_push(ary,to_ruby(what[1])); // length of text
      return ary;
      }
    else if(type==SEL_REPLACED) {
      FXint* what=(FXint*)ptr;
      VALUE ary=rb_ary_new();
      rb_ary_push(ary,to_ruby(what[0])); // start position for replace
      rb_ary_push(ary,to_ruby(what[1])); // length of old text
      rb_ary_push(ary,to_ruby(what[2])); // length of new text
      return ary;
      }
    }
  else if(sender->isMemberOf(FXMETACLASS(FXTextField))){
    if(type==SEL_CHANGED ||
       type==SEL_COMMAND ||
       type==SEL_VERIFY) return to_ruby((const FXchar*)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXToggleButton))){
    if(type==SEL_COMMAND) return to_ruby((FXuchar)(long)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXToolbarTab))){
    if (type==SEL_CLICKED ||
        type==SEL_DOUBLECLICKED ||
        type==SEL_TRIPLECLICKED ||
        type==SEL_COMMAND) return to_ruby((FXbool)(long)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXTreeList))){
    if(type==SEL_COLLAPSED ||
       type==SEL_EXPANDED ||
       type==SEL_COMMAND ||
       type==SEL_CHANGED ||
       type==SEL_CLICKED ||
       type==SEL_DOUBLECLICKED ||
       type==SEL_TRIPLECLICKED ||
       type==SEL_OPENED ||
       type==SEL_CLOSED){
      return FXRbWrapClassInstance(cFX_TreeItem,ptr);
      }
    else if(type==SEL_SELECTED ||
            type==SEL_DESELECTED) return to_ruby((FXTreeItem**)ptr);
    }
  else if(sender->isMemberOf(FXMETACLASS(FXTreeListBox))){
    if(type==SEL_CHANGED || type==SEL_COMMAND)
      return FXRbWrapClassInstance(cFX_TreeItem,ptr);
    }
  else{
    return to_ruby(ptr); // FIXME (no matching message type?)
    }
  return to_ruby(ptr); // FIXME (no matching message type?)
  }


/**
 * When a Ruby instance (e.g. one of your widgets) calls handle() on another
 * object, e.g.
 *
 *     def onUpdAnswer(sender, sel, ptr)
 *       if theAnswerIsTwo?
 *         sender.handle(self, MKUINT(FXWindow::ID_SETVALUE,SEL_COMMAND), 2)
 *       else
 *         sender.handle(self, MKUINT(FXWindow::ID_SETVALUE,SEL_COMMAND), 3)
 *       end
 *       return 1
 *     end
 *
 * it's usually the case that this will get passed along to the underlying
 * C++ object. In that case, we need to convert the message data (e.g. the
 * Fixnums "2" or "3" in the example above) from Ruby objects back into
 * the appropriate C++ objects. That's what this function is for.
 */
void* FXRbGetExpectedData(VALUE recv,FXSelector key,VALUE value){
  FXEvent* ev;
  static FXint intValue;
  static FXint intRange[2];
  static FXdouble realValue;
  static FXdouble realRange[2];
  static FXString stringValue;
  static FXColor colorValue;
  FXint type=SELTYPE(key);
  FXint id=SELID(key);
  switch(type){
    case SEL_KEYPRESS:
    case SEL_KEYRELEASE:
    case SEL_LEFTBUTTONPRESS:
    case SEL_LEFTBUTTONRELEASE:
    case SEL_MIDDLEBUTTONPRESS:
    case SEL_MIDDLEBUTTONRELEASE:
    case SEL_RIGHTBUTTONPRESS:
    case SEL_RIGHTBUTTONRELEASE:
    case SEL_MOTION:
    case SEL_ENTER:
    case SEL_LEAVE:
    case SEL_FOCUSIN:
    case SEL_FOCUSOUT:
    case SEL_KEYMAP:
    case SEL_UNGRABBED:
    case SEL_PAINT:
    case SEL_CREATE:
    case SEL_DESTROY:
    case SEL_UNMAP:
    case SEL_MAP:
    case SEL_CONFIGURE:
    case SEL_SELECTION_LOST:
    case SEL_SELECTION_GAINED:
    case SEL_SELECTION_REQUEST:
    case SEL_RAISED:
    case SEL_LOWERED:
    case SEL_MOUSEWHEEL:
    case SEL_BEGINDRAG:
    case SEL_ENDDRAG:
    case SEL_DRAGGED:
    case SEL_LASSOED:
    case SEL_TIMEOUT:
    case SEL_CLIPBOARD_LOST:
    case SEL_CLIPBOARD_GAINED:
    case SEL_CLIPBOARD_REQUEST:
    case SEL_CHORE:
    case SEL_FOCUS_SELF:
    case SEL_FOCUS_RIGHT:
    case SEL_FOCUS_LEFT:
    case SEL_FOCUS_DOWN:
    case SEL_FOCUS_UP:
    case SEL_FOCUS_NEXT:
    case SEL_FOCUS_PREV:
    case SEL_DND_ENTER:
    case SEL_DND_LEAVE:
    case SEL_DND_DROP:
    case SEL_DND_MOTION:
    case SEL_DND_REQUEST:
    case SEL_PICKED:
      Get_FXEvent(value,ev);
      return ev;
    case SEL_IO_READ:
    case SEL_IO_WRITE:
    case SEL_IO_EXCEPT:
      return NULL; // should be an FXInputHandle?
    case SEL_SIGNAL:
      return (void*) NUM2INT(value);
    case SEL_CLOSE:
    case SEL_CLOSEALL:
    case SEL_DELETE:
    case SEL_MINIMIZE:
    case SEL_RESTORE:
    case SEL_MAXIMIZE:
    case SEL_UPDATE:
      return NULL;
    case SEL_VERIFY:
      return (void*) STR2CSTR(value);
    case SEL_UNCHECK_OTHER:
    case SEL_UNCHECK_RADIO:
      return NULL; // ??? unused
    case SEL_CLICKED:
    case SEL_DOUBLECLICKED:
    case SEL_TRIPLECLICKED:
    case SEL_CHANGED:
    case SEL_DESELECTED:
    case SEL_SELECTED:
    case SEL_INSERTED:
    case SEL_REPLACED:
    case SEL_DELETED:
    case SEL_OPENED:
    case SEL_CLOSED:
    case SEL_EXPANDED:
    case SEL_COLLAPSED:
      return NULL;
    default:
      /* Ignore */
      break;
    }
  if(type==SEL_COMMAND){
    if(rb_obj_is_kind_of(recv,cFX_Window)){
      switch(id){
        case FXWindow::ID_HIDE:
        case FXWindow::ID_SHOW:
        case FXWindow::ID_TOGGLESHOWN:
        case FXWindow::ID_LOWER:
        case FXWindow::ID_RAISE:
        case FXWindow::ID_DELETE:
        case FXWindow::ID_DISABLE:
        case FXWindow::ID_ENABLE:
        case FXWindow::ID_UNCHECK:
        case FXWindow::ID_CHECK:
        case FXWindow::ID_UNKNOWN:
        case FXWindow::ID_UPDATE:
        case FXWindow::ID_AUTOSCROLL:
        case FXWindow::ID_QUERY_TIP:
        case FXWindow::ID_QUERY_HELP:
        case FXWindow::ID_QUERY_MENU:
        case FXWindow::ID_HOTKEY:
        case FXWindow::ID_ACCEL:
        case FXWindow::ID_UNPOST:
        case FXWindow::ID_POST:
        case FXWindow::ID_MDI_TILEHORIZONTAL:
        case FXWindow::ID_MDI_TILEVERTICAL:
        case FXWindow::ID_MDI_CASCADE:
        case FXWindow::ID_MDI_MAXIMIZE:
        case FXWindow::ID_MDI_MINIMIZE:
        case FXWindow::ID_MDI_RESTORE:
        case FXWindow::ID_MDI_CLOSE:
        case FXWindow::ID_MDI_WINDOW:
        case FXWindow::ID_MDI_MENUWINDOW:
        case FXWindow::ID_MDI_MENUMINIMIZE:
        case FXWindow::ID_MDI_MENURESTORE:
        case FXWindow::ID_MDI_MENUCLOSE:
        case FXWindow::ID_MDI_NEXT:
        case FXWindow::ID_MDI_PREV:
        case FXWindow::ID_CLOSE_DOCUMENT:
        case FXWindow::ID_CLOSE_ALL_DOCUMENTS:
          return NULL;
        case FXWindow::ID_SETVALUE:
          if((rb_obj_is_kind_of(recv,cFX_Button)==Qtrue) ||
             (rb_obj_is_kind_of(recv,cFX_CheckButton)==Qtrue) ||
             (rb_obj_is_kind_of(recv,cFX_Dial)==Qtrue) ||
             (rb_obj_is_kind_of(recv,cFX_RadioButton)==Qtrue) ||
             (rb_obj_is_kind_of(recv,cFX_Shutter)==Qtrue) ||
             (rb_obj_is_kind_of(recv,cFX_Spinner)==Qtrue) ||
             (rb_obj_is_kind_of(recv,cFX_TabBar)==Qtrue) ||
             (rb_obj_is_kind_of(recv,cFX_ToggleButton)==Qtrue) ||
             (rb_obj_is_kind_of(recv,cFX_Scrollbar)==Qtrue) ||
             (rb_obj_is_kind_of(recv,cFX_Slider)==Qtrue) ||
             (rb_obj_is_kind_of(recv,cFX_Switcher)==Qtrue)){
            return (void*) NUM2INT(value);
            }
          else if((rb_obj_is_kind_of(recv,cFX_ColorSelector)==Qtrue) ||
                  (rb_obj_is_kind_of(recv,cFX_ColorWell)==Qtrue)){
            return (void*) NUM2UINT(value);
            }
          else if(rb_obj_is_kind_of(recv,cFX_ProgressBar)==Qtrue){
            return (void*) NUM2UINT(value);
            }
          else if((rb_obj_is_kind_of(recv,cFX_ComboBox)==Qtrue) ||
                  (rb_obj_is_kind_of(recv,cFX_TextField)==Qtrue) ||
                  (rb_obj_is_kind_of(recv,cFX_DirBox)==Qtrue) ||
                  (rb_obj_is_kind_of(recv,cFX_DirList)==Qtrue) ||
                  (rb_obj_is_kind_of(recv,cFX_DriveBox)==Qtrue) ||
                  (rb_obj_is_kind_of(recv,cFX_FileList)==Qtrue)){
            return (void*) STR2CSTR(value);
            }
          else if(rb_obj_is_kind_of(recv,cFX_MenuCommand)==Qtrue){
            return RTEST(value)?(void*)1:(void*)0;
            }
          return NULL;
        case FXWindow::ID_HSCROLLED:
        case FXWindow::ID_VSCROLLED:
          return (void*) NUM2INT(value);
        case FXWindow::ID_SETINTVALUE:
          if(rb_obj_is_kind_of(recv,cFX_ColorWell)==Qtrue){
            colorValue=NUM2UINT(value);
            return &colorValue;
            }
          else{
            intValue=NUM2INT(value);
            return &intValue;
            }
        case FXWindow::ID_SETREALVALUE:
          realValue=NUM2DBL(value);
          return &realValue;
        case FXWindow::ID_SETSTRINGVALUE:
          stringValue=FXString(STR2CSTR(value));
          return &stringValue;
        case FXWindow::ID_SETINTRANGE:
          intRange[0]=NUM2INT(rb_ary_entry(value,0));
          intRange[1]=NUM2INT(rb_ary_entry(value,1));
          return intRange;
        case FXWindow::ID_SETREALRANGE:
          realRange[0]=NUM2DBL(rb_ary_entry(value,0));
          realRange[1]=NUM2DBL(rb_ary_entry(value,1));
          return realRange;
        case FXWindow::ID_GETINTVALUE:
        case FXWindow::ID_GETREALVALUE:
        case FXWindow::ID_GETSTRINGVALUE:
        case FXWindow::ID_GETINTRANGE:
        case FXWindow::ID_GETREALRANGE:
          return NULL;
        default:
          /* Ignore */
          break;
        }
      }
    }
  return NULL;
  }


/**
 * Look up the name of the message handler function for this
 * receiver and message (type, id) combination and return it;
 * or return NULL if the designated receiver doesn't handle this
 * message.
 */
const char* FXRbLookupHandler(FXObject* recv,FXSelector key){
  VALUE rubyObj=to_ruby(recv);
  ID id_assocs=rb_intern("@assocs");
  if(rb_ivar_defined(rubyObj,id_assocs)==Qtrue){
    VALUE assocs=rb_ivar_get(rubyObj,id_assocs);
    Check_Type(assocs,T_ARRAY);
    VALUE entry;
    FXSelector keylo,keyhi;
    for(long i=0;i<RARRAY(assocs)->len;i++){
      entry=rb_ary_entry(assocs,i);
      Check_Type(entry,T_ARRAY);
      keylo=NUM2UINT(rb_ary_entry(entry,0));
      keyhi=NUM2UINT(rb_ary_entry(entry,1));
      if(keylo<=key && key<=keyhi){
        const char* func=STR2CSTR(rb_ary_entry(entry,2));
        FXTRACE((100,"FXRbLookupHandler(%s(%p)): returns \"%s\"\n",recv->getClassName(),recv,func));
        return func;
        }
      }
    }
  return NULL;
  }


struct FXRbHandleArgs {
  VALUE recv;
  VALUE id;
  int nargs;
  VALUE sender;
  VALUE key;
  VALUE data;
  };


static VALUE handle_body(VALUE args){
  FXRbHandleArgs* hArgs=(FXRbHandleArgs*) args;
  FXASSERT(hArgs!=0);
  return rb_funcall(hArgs->recv,hArgs->id,hArgs->nargs,hArgs->sender,hArgs->key,hArgs->data);
  }


static VALUE handle_rescue(VALUE args){
  VALUE info=rb_gv_get("$!");
  VALUE errat=rb_funcall(info,rb_intern("backtrace"),0);
  VALUE mesg=RARRAY(errat)->ptr[0];
  fprintf(stderr,"%s: %s (%s)\n",
    STR2CSTR(mesg),
    STR2CSTR(rb_obj_as_string(info)),
    rb_class2name(CLASS_OF(info)));
  for(int i=1;i<RARRAY(errat)->len;i++){
    if(TYPE(RARRAY(errat)->ptr[i])==T_STRING){
      fprintf(stderr,"\tfrom %s\n",STR2CSTR(RARRAY(errat)->ptr[i]));
      }
    }
  return Qnil;
  }


// Call the designated function and return its result (which should be a long).
long FXRbHandleMessage(FXObject* recv,const char* func,FXObject* sender,FXSelector key,void* ptr){
  FXTRACE((100,"FXRbHandleMessage(%s(%p), \"%s\")\n",recv->getClassName(),recv,func)); 
  FXRbHandleArgs hArgs;
  hArgs.recv=to_ruby(recv);
  FXTRACE((100,"  to_ruby(recv)=%d\n",hArgs.recv));
  hArgs.sender=to_ruby(sender);
  FXTRACE((100,"  to_ruby(sender)=%d\n",hArgs.sender));
  hArgs.key=to_ruby(key);
  FXTRACE((100,"  to_ruby(key)=%d\n",hArgs.key));
  hArgs.data=FXRbConvertMessageData(sender,key,ptr);
  FXTRACE((100,"  rbMsgData=%d\n",hArgs.data));
  hArgs.id=rb_intern(func);
  hArgs.nargs=3;
  VALUE retval;
  if(FXRbCatchExceptions){
    retval=rb_rescue2(handle_body,(VALUE)&hArgs,handle_rescue,Qnil,
      rb_eStandardError,rb_eNameError,0);
    }
  else{
    retval=handle_body((VALUE)&hArgs);
    }
  FXTRACE((100,"  rb_funcall(\"%s\") returned VALUE=%d\n",func,retval));

  /**
   * Process the return value. For boolean return values, convert "true"
   * to 1 and "false" to zero. For numeric types, convert it to a long value
   * but trap the result to either 0 or 1 since these are usually what
   * FOX is looking for. For any other type result (including nil) return 1.
   * Thanks to Ted Meng for this suggestion.
   */
  long lresult=0;
  switch(TYPE(retval)){
    case T_TRUE:
      return 1;
    case T_FALSE:
      return 0;
    case T_FIXNUM:
    case T_BIGNUM:
    case T_FLOAT:
      lresult=NUM2LONG(retval);
      return (lresult==0)?0:1; // trap any numeric result to either 0 or 1
    default:
      return 1;
    }
  }

//----------------------------------------------------------------------

void FXRbCallVoidMethod(FXObject* recv, const char *func) {
  /*
  FXTRACE((100,"start FXRbCallVoidMethod(%p (%s), \"%s\")\n",
    recv,recv->getClassName(),func));
  */
  FXTRACE((100,"start FXRbCallVoidMethod(%p, \"%s\")\n",
    recv,func));
  rb_funcall(FXRbGetRubyObj(recv), rb_intern(func), 0, NULL);
  FXTRACE((100,"after rb_funcall(\"%s\")\n",func));
  }

void FXRbCallVoidMethod(FXDC* recv, const char *func) {
  rb_funcall(FXRbGetRubyObj(recv), rb_intern(func), 0, NULL);
  }

//----------------------------------------------------------------------

FXbool FXRbCallBoolMethod(const FXObject* recv,const char* func){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),0,NULL);
  return (v==Qtrue)?TRUE:FALSE;
  }

//----------------------------------------------------------------------

// Call function with "FXint" return value
FXint FXRbCallIntMethod(FXObject* recv,const char* func){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),0,NULL);
  return (FXint) NUM2INT(v);
  }

//----------------------------------------------------------------------

// Call function with "FXGLObject*" return value
FXGLObject* FXRbCallGLObjectMethod(FXObject* recv,const char* func){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),0,NULL);
  return (FXGLObject*) DATA_PTR(v);
  }

//----------------------------------------------------------------------

// Call function with "FXRange" return value
FXRange FXRbCallRangeMethod(FXObject* recv,const char* func){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),0,NULL);
  FXRange box=*((FXRange*) DATA_PTR(v));
  return box;
  }

//----------------------------------------------------------------------

// Special destructor to handle order dependencies
FXRbMenuCommand::~FXRbMenuCommand(){
  FXAccelTable *table;
  FXWindow *owner;
  if(acckey){
    owner=getShell()->getOwner();
    if(owner){
      table=owner->getAccelTable();
      if(table && table!=(FXAccelTable*)-1){
        table->removeAccel(acckey);
        }
      }
    }
  acckey=(FXHotKey)NULL;
  FXRbUnregisterRubyObj(this);
  }

//----------------------------------------------------------------------

// Sort function stand-in for FXIconList
FXint FXRbIconList::sortProc(const FXIconItem* a,const FXIconItem* b){
  return 0;
  }


// Sort function stand-in for FXList
FXint FXRbList::sortProc(const FXListItem* a,const FXListItem* b){
  return 0;
  }


// Sort function stand-in for FXTreeList
FXint FXRbTreeList::sortProc(const FXTreeItem* a,const FXTreeItem* b){
  return 0;
  }


// Feedback buffer sort routine stand-in for FXGLViewer
FXbool FXRbGLViewer::sortProc(FXfloat*& buffer,FXint& used,FXint& size){
  return TRUE;
  }

//----------------------------------------------------------------------

// Copied from the Ruby 1.6.6 sources (signal.c)
static struct signals {
  const char* signm;
  FXint signo;
  } siglist[]={
#ifdef SIGHUP
    "HUP", SIGHUP,
#endif
#ifdef SIGINT
    "INT", SIGINT,
#endif
#ifdef SIGQUIT
    "QUIT", SIGQUIT,
#endif
#ifdef SIGILL
    "ILL", SIGILL,
#endif
#ifdef SIGTRAP
    "TRAP", SIGTRAP,
#endif
#ifdef SIGIOT
    "IOT", SIGIOT,
#endif
#ifdef SIGABRT
    "ABRT", SIGABRT,
#endif
#ifdef SIGEMT
    "EMT", SIGEMT,
#endif
#ifdef SIGFPE
    "FPE", SIGFPE,
#endif
#ifdef SIGKILL
    "KILL", SIGKILL,
#endif
#ifdef SIGBUS
    "BUS", SIGBUS,
#endif
#ifdef SIGSEGV
    "SEGV", SIGSEGV,
#endif
#ifdef SIGSYS
    "SYS", SIGSYS,
#endif
#ifdef SIGPIPE
    "PIPE", SIGPIPE,
#endif
#ifdef SIGALRM
    "ALRM", SIGALRM,
#endif
#ifdef SIGTERM
    "TERM", SIGTERM,
#endif
#ifdef SIGURG
    "URG", SIGURG,
#endif
#ifdef SIGSTOP
    "STOP", SIGSTOP,
#endif
#ifdef SIGTSTP
    "TSTP", SIGTSTP,
#endif
#ifdef SIGCONT
    "CONT", SIGCONT,
#endif
#ifdef SIGCHLD
    "CHLD", SIGCHLD,
#endif
#ifdef SIGCLD
    "CLD", SIGCLD,
#else
# ifdef SIGCHLD
    "CLD", SIGCHLD,
# endif
#endif
#ifdef SIGTTIN
    "TTIN", SIGTTIN,
#endif
#ifdef SIGTTOU
    "TTOU", SIGTTOU,
#endif
#ifdef SIGIO
    "IO", SIGIO,
#endif
#ifdef SIGXCPU
    "XCPU", SIGXCPU,
#endif
#ifdef SIGXFSZ
    "XFSZ", SIGXFSZ,
#endif
#ifdef SIGVTALRM
    "VTALRM", SIGVTALRM,
#endif
#ifdef SIGPROF
    "PROF", SIGPROF,
#endif
#ifdef SIGWINCH
    "WINCH", SIGWINCH,
#endif
#ifdef SIGUSR1
    "USR1", SIGUSR1,
#endif
#ifdef SIGUSR2
    "USR2", SIGUSR2,
#endif
#ifdef SIGLOST
    "LOST", SIGLOST,
#endif
#ifdef SIGMSG
    "MSG", SIGMSG,
#endif
#ifdef SIGPWR
    "PWR", SIGPWR,
#endif
#ifdef SIGPOLL
    "POLL", SIGPOLL,
#endif
#ifdef SIGDANGER
    "DANGER", SIGDANGER,
#endif
#ifdef SIGMIGRATE
    "MIGRATE", SIGMIGRATE,
#endif
#ifdef SIGPRE
    "PRE", SIGPRE,
#endif
#ifdef SIGGRANT
    "GRANT", SIGGRANT,
#endif
#ifdef SIGRETRACT
    "RETRACT", SIGRETRACT,
#endif
#ifdef SIGSOUND
    "SOUND", SIGSOUND,
#endif
#ifdef SIGINFO
    "INFO", SIGINFO,
#endif
    NULL, 0,
};

FXint FXRbSignalNameToNumber(const char* s){
#ifdef HAVE_SIGNAL_H
  char *nm=(char*) s;
  if(strncmp("SIG",nm,3)==0){
    nm+=3;
    }
  for(signals* sigs=siglist;sigs->signm;sigs++){
    if(strcmp(sigs->signm,nm)==0)
      return sigs->signo;
    }
#endif /* HAVE_SIGNAL_H */
  return 0;
  }

//----------------------------------------------------------------------

extern "C" void Init_core(void);
extern "C" void Init_dialogs(void);
extern "C" void Init_icons(void);
extern "C" void Init_image(void);
extern "C" void Init_mdi(void);
extern "C" void Init_opengl(void);
#ifdef WITH_FXSCINTILLA
extern "C" void Init_scintilla(void);
#endif

extern "C" void Init_fox(void) {
  Init_core();
  Init_dialogs();
  Init_icons();
  Init_image();
  Init_mdi();
  Init_opengl();
#ifdef WITH_FXSCINTILLA
  Init_scintilla();
#endif
  rb_f_require(Qnil, rb_str_new2("fox/aliases"));
  rb_f_require(Qnil, rb_str_new2("fox/core"));
  rb_f_require(Qnil, rb_str_new2("fox/iterators"));
  rb_f_require(Qnil, rb_str_new2("fox/keys"));
  rb_f_require(Qnil, rb_str_new2("fox/overloads"));
  rb_f_require(Qnil, rb_str_new2("fox/responder2"));
  }
