/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: FXRuby.h,v 1.9 2002/01/03 00:52:20 ljohnson Exp $
 ***********************************************************************/

#ifndef FXRUBY_H
#define FXRUBY_H

// Returns an FXInputHandle for this Ruby file object
FXInputHandle FXRbGetInputHandle(VALUE obj);

// Register mapping from Ruby objects to FOX objects
void FXRbRegisterRubyObj(VALUE rubyObj, const void* foxObj);

// Remove mapping for this FOX object
void FXRbUnregisterRubyObj(const void* foxObj);

// Register an object that must be destroyed before FXApp is destroyed
void FXRbRegisterAppSensitiveObject(FXObject* obj);

// Unregister an object that must be destroyed before FXApp is destroyed
void FXRbUnregisterAppSensitiveObject(FXObject* obj);

// Destroy all app-sensitive objects
void FXRbDestroyAppSensitiveObjects();

// Return the Ruby class instance associated with this FOX object
VALUE FXRbGetRubyObj(const void *foxObj);

// Mark this object as used for the garbage collector
void FXRbGcMark(void *obj);

// Construct a Ruby data object to hold a data pointer
extern "C" VALUE FXRbWrapVoidPointer(void* ptr);

// Extract data pointer from a Ruby data object
extern "C" void* FXRbGetVoidPointer(VALUE obj);

// Return a wrapped class instance
VALUE FXRbWrapClassInstance(VALUE module,const char* classname,void* ptr);

// Return a wrapped class instance
VALUE FXRbWrapClassInstance(VALUE klass,void* ptr);

// Returns a Ruby array of floats
extern VALUE FXRbMakeArray(const FXfloat* values,FXint size);

// Returns a Ruby array of floats
extern VALUE FXRbMakeArray(const FXdouble* values,FXint size);

// Returns a Ruby array of integers
extern VALUE FXRbMakeArray(const FXint* values,FXint size);

extern void* FXRbGetExpectedData(VALUE recv,FXSelector key,VALUE data);

// Returns the name of the message handler function (or NULL)
const char* FXRbLookupHandler(FXObject* recv,FXSelector key);

// Handle this message
long FXRbHandleMessage(FXObject* recv,const char* func,FXObject* sender,FXSelector key,void* data);

// Convert a signal name string to its corresponding signal number
FXint FXRbSignalNameToNumber(const char* name);

// Fox module instance
extern VALUE mFox;

// Convert from FOX datatypes to Ruby objects
inline VALUE to_ruby(FXchar c){
  return INT2NUM(c);
  }

inline VALUE to_ruby(FXbool b){ // same as FXuchar
  return b ? Qtrue : Qfalse;
  }

inline VALUE to_ruby(FXshort s){
  return INT2NUM(s);
  }

inline VALUE to_ruby(FXushort u){
  return UINT2NUM(u);
  }

inline VALUE to_ruby(FXint i){
  return INT2NUM(i);
  }

inline VALUE to_ruby(FXuint u){
  return UINT2NUM(u);
  }

inline VALUE to_ruby(FXfloat f){
  return rb_float_new(f);
  }

inline VALUE to_ruby(FXdouble d){
  return rb_float_new(d);
  }

inline VALUE to_ruby(unsigned long l){
  return UINT2NUM(l);
  }

inline VALUE to_ruby(const FXString& s){
  return rb_str_new2(s.text());
  }

inline VALUE to_ruby(const FXchar* s){
  return rb_str_new2(s);
  }

inline VALUE to_ruby(FXObject* obj){
  return FXRbGetRubyObj(obj);
  }

inline VALUE to_ruby(const FXObject* obj){
  return FXRbGetRubyObj(obj);
  }

inline VALUE to_ruby(const FXRange& range){
  return FXRbWrapClassInstance(mFox,"FXRange",(void*)&range);
  }

inline VALUE to_ruby(void* ptr){
  return FXRbWrapVoidPointer(ptr);
  }

inline VALUE to_ruby(FXStream& store){
  return (VALUE) 0; // FIXME
  }

inline VALUE to_ruby(const FXPoint* point){
  return FXRbWrapClassInstance(mFox,"FXPoint",(void*)point);
  }
 
inline VALUE to_ruby(const FXSegment* segment){
  return FXRbWrapClassInstance(mFox,"FXSegment",(void*)segment);
  }

inline VALUE to_ruby(const FXRectangle* rect){
  return FXRbWrapClassInstance(mFox,"FXRectangle",(void*)rect);
  }

inline VALUE to_ruby(const FXRectangle& rect){
  return FXRbWrapClassInstance(mFox,"FXRectangle",(void*)&rect);
  }

inline VALUE to_ruby(const FXArc* arc){
  return FXRbWrapClassInstance(mFox,"FXArc",(void*)arc);
  }

inline VALUE to_ruby(const FXRegion& region){
  return (VALUE) 0; // FIXME
  }

inline VALUE to_ruby(FXEvent* event){
  return FXRbWrapClassInstance(mFox,"FXEvent",(void*)event);
  }

inline VALUE to_ruby(FXTablePos* p){
  return FXRbWrapClassInstance(mFox,"FXTablePos",(void*)p);
  }

inline VALUE to_ruby(FXTableRange* r){
  return FXRbWrapClassInstance(mFox,"FXTableRange",(void*)r);
  }

inline VALUE to_ruby(FXTreeItem** ppi){
  return (VALUE) 0; // FIXME
  }

inline VALUE to_ruby(FXFontDesc* fontdesc){
  return FXRbWrapClassInstance(mFox,"FXFontDesc",(void*)fontdesc);
  }

// Call function with "void" return value
void FXRbCallVoidMethod(FXObject* recv,
                        const char* func);

void FXRbCallVoidMethod(FXDC* recv,
                        const char* func);

template<class TYPE>
void FXRbCallVoidMethod(FXObject* recv, const char* func, TYPE arg){
  rb_funcall(FXRbGetRubyObj(recv), rb_intern(func), 1, to_ruby(arg));
  }

template<class TYPE>
void FXRbCallVoidMethod(FXDC* recv, const char* func, TYPE arg){
  rb_funcall(FXRbGetRubyObj(recv), rb_intern(func), 1, to_ruby(arg));
  }
 
template<class TYPE1, class TYPE2>
void FXRbCallVoidMethod(FXObject* recv, const char* func, TYPE1 arg1, TYPE2 arg2){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),2,to_ruby(arg1),to_ruby(arg2));
  }

template<class TYPE1, class TYPE2>
void FXRbCallVoidMethod(FXDC* recv, const char* func, TYPE1 arg1, TYPE2 arg2){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),2,to_ruby(arg1),to_ruby(arg2));
  }

template<class TYPE1, class TYPE2, class TYPE3>
void FXRbCallVoidMethod(FXObject* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),3,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3));
  }

template<class TYPE1, class TYPE2, class TYPE3>
void FXRbCallVoidMethod(FXDC* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),3,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3));
  }

template<class TYPE1, class TYPE2, class TYPE3, class TYPE4>
void FXRbCallVoidMethod(FXObject* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3, TYPE4 arg4){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),4,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3),to_ruby(arg4));
  }

template<class TYPE1, class TYPE2, class TYPE3, class TYPE4>
void FXRbCallVoidMethod(FXDC* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3, TYPE4 arg4){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),4,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3),to_ruby(arg4));
  }

template<class TYPE1, class TYPE2, class TYPE3, class TYPE4, class TYPE5>
void FXRbCallVoidMethod(FXObject* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3, TYPE4 arg4, TYPE5 arg5){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),5,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3),to_ruby(arg4),to_ruby(arg5));
  }

template<class TYPE1, class TYPE2, class TYPE3, class TYPE4, class TYPE5>
void FXRbCallVoidMethod(FXDC* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3, TYPE4 arg4, TYPE5 arg5){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),5,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3),to_ruby(arg4),to_ruby(arg5));
  }

template<class TYPE1, class TYPE2, class TYPE3, class TYPE4, class TYPE5, class TYPE6>
void FXRbCallVoidMethod(FXDC* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3, TYPE4 arg4, TYPE5 arg5, TYPE6 arg6){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),6,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3),to_ruby(arg4),to_ruby(arg5),to_ruby(arg6));
  }

template<class TYPE1, class TYPE2, class TYPE3, class TYPE4, class TYPE5, class TYPE6, class TYPE7>
void FXRbCallVoidMethod(FXDC* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3, TYPE4 arg4, TYPE5 arg5, TYPE6 arg6, TYPE7 arg7){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),7,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3),to_ruby(arg4),to_ruby(arg5),to_ruby(arg6),to_ruby(arg7));
  }

template<class TYPE>
void FXRbCallVoidMethod(const FXObject* recv, const char* func, TYPE arg){
  rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),1,to_ruby(arg));
  }

// Call function with "FXbool" return value
template<class TYPE>
FXbool FXRbCallBoolMethod(FXStream* recv, const char* func, TYPE arg){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),1,to_ruby(arg));
  return (v==Qtrue)?TRUE:FALSE;
  }

FXbool FXRbCallBoolMethod(const FXObject* recv, const char* func);

template<class TYPE>
FXbool FXRbCallBoolMethod(FXObject* recv, const char* func, TYPE arg){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),1,to_ruby(arg));
  return (v==Qtrue)?TRUE:FALSE;
  }

template<class TYPE1, class TYPE2>
FXbool FXRbCallBoolMethod(const FXObject* recv, const char* func, TYPE1 arg1, TYPE2 arg2){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),2,to_ruby(arg1),to_ruby(arg2));
  return (v==Qtrue)?TRUE:FALSE;
  }

template<class TYPE1, class TYPE2, class TYPE3>
FXbool FXRbCallBoolMethod(const FXObject* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),3,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3));
  return (v==Qtrue)?TRUE:FALSE;
  }

template<class TYPE1, class TYPE2, class TYPE3, class TYPE4, class TYPE5>
FXbool FXRbCallBoolMethod(const FXObject* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3, TYPE4 arg4, TYPE5 arg5){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),5,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3),to_ruby(arg4),to_ruby(arg5));
  return (v==Qtrue)?TRUE:FALSE;
  }

// Call function with "FXint" return value
FXint FXRbCallIntMethod(FXObject* recv, const char* func);

template<class TYPE>
FXint FXRbCallIntMethod(FXObject* recv, const char* func, TYPE arg){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),1,to_ruby(arg));
  return (FXint) NUM2INT(v);
  }

template<class TYPE>
FXint FXRbCallIntMethod(const FXObject* recv, const char* func, TYPE arg){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),1,to_ruby(arg));
  return (FXint) NUM2INT(v);
  }

// Call function with "long" return value
template<class TYPE1, class TYPE2, class TYPE3>
long FXRbCallLongMethod(FXObject* recv, const char* func, TYPE1 arg1, TYPE2 arg2, TYPE3 arg3){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),3,to_ruby(arg1),to_ruby(arg2),to_ruby(arg3));
  return (long) NUM2LONG(v);
  }

// Call functions with "FXuint" return value
template<class TYPE>
FXuint FXRbCallUIntMethod(FXObject* recv, const char* func, TYPE arg){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),1,to_ruby(arg));
  return (FXuint) NUM2UINT(v);
  }



// Call functions with "FXGLObject*" return value
FXGLObject* FXRbCallGLObjectMethod(FXObject* recv, const char* func);


// Call functions with "FXColor" return value
template<class TYPE1, class TYPE2>
FXColor FXRbCallColorMethod(FXDC* recv, const char* func, TYPE1 arg1, TYPE2 arg2){
  VALUE v=rb_funcall(FXRbGetRubyObj(recv),rb_intern(func),2,to_ruby(arg1),to_ruby(arg2));
  return (FXColor) NUM2UINT(v);
  }

// Call functions with "FXRange" return value
FXRange FXRbCallRangeMethod(FXObject* recv, const char* func);

/**
 * Macro to set up class implementation.
 * This is a modified version of the standard FOX macro FXIMPLEMENT();
 * this version's implementation of the virtual handle() function
 * first checks the Ruby class instance's message map to see if it
 * can handle the incoming message. If not, the regular C++ message
 * mapping takes over as in standard FOX applications.
 */

#define FXRbIMPLEMENT(classname,baseclassname,mapping,nmappings) \
  FXObject* classname::manufacture(){return new classname;} \
  const FXMetaClass classname::metaClass={#classname,classname::manufacture,&baseclassname::metaClass,mapping,nmappings,sizeof(classname::FXMapEntry),sizeof(#classname)}; \
  __FXMETACLASSINITIALIZER__ classname##Initializer(&classname::metaClass); \
  long classname::handle(FXObject* sender,FXSelector key,void* ptr){ \
    const char* func=FXRbLookupHandler(this,key); \
    if(func){ \
      return FXRbHandleMessage(this,func,sender,key,ptr); \
      } \
    else{ \
      const FXMapEntry* me=(const FXMapEntry*)metaClass.search(key); \
      return me ? (this->* me->func)(sender,key,ptr) : baseclassname::handle(sender,key,ptr); \
      } \
    }


// FXRuby classes
#include "FXRbStream.h"
#include "FXRbObject.h"
#include "FXRbDelegator.h"
#include "FXRbDict.h"
#include "FXRbStringDict.h"
#include "FXRbAccelTable.h"
#include "FXRbRecentFiles.h"
#include "FXRbApp.h"
#include "FXRbId.h"
#include "FXRbVisual.h"
#include "FXRbFont.h"
#include "FXRbCursor.h"
#include "FXRbGIFCursor.h"
#include "FXRbDrawable.h"
#include "FXRbBitmap.h"
#include "FXRbImage.h"
#include "FXRbBMPImage.h"
#include "FXRbGIFImage.h"
#include "FXRbICOImage.h"
#include "FXRbJPGImage.h"
#include "FXRbPCXImage.h"
#include "FXRbPNGImage.h"
#include "FXRbTGAImage.h"
#include "FXRbTIFImage.h"
#include "FXRbXPMImage.h"
#include "FXRbIcon.h"
#include "FXRbBMPIcon.h"
#include "FXRbGIFIcon.h"
#include "FXRbICOIcon.h"
#include "FXRbJPGIcon.h"
#include "FXRbPCXIcon.h"
#include "FXRbPNGIcon.h"
#include "FXRbTGAIcon.h"
#include "FXRbTIFIcon.h"
#include "FXRbXPMIcon.h"
#include "FXRbDC.h"
#include "FXRbDCWindow.h"
#include "FXRbDCPrint.h"
#include "FXRbFileDict.h"
#include "FXRbWindow.h"
#include "FXRbFrame.h"
#include "FXRbSeparator.h"
#include "FXRbLabel.h"
#include "FXRbDial.h"
#include "FXRbColorBar.h"
#include "FXRbColorWell.h"
#include "FXRbColorWheel.h"
#include "FXRbTextField.h"
#include "FXRbButton.h"
#include "FXRbPicker.h"
#include "FXRbToggleButton.h"
#include "FXRbCheckButton.h"
#include "FXRbRadioButton.h"
#include "FXRbArrowButton.h"
#include "FXRbMenuButton.h"
#include "FXRbComposite.h"
#include "FXRbPacker.h"
#include "FXRbHorizontalFrame.h"
#include "FXRbVerticalFrame.h"
#include "FXRbMatrix.h"
#include "FXRbSpinner.h"
#include "FXRbRootWindow.h"
#include "FXRbCanvas.h"
#include "FXRbGroupBox.h"
#include "FXRbShell.h"
#include "FXRbTooltip.h"
#include "FXRbPopup.h"
#include "FXRbTopWindow.h"
#include "FXRbDialogBox.h"
#include "FXRbMainWindow.h"
#include "FXRbMenuPane.h"
#include "FXRbMenuCaption.h"
#include "FXRbMenuSeparator.h"
#include "FXRbMenuTitle.h"
#include "FXRbMenuCascade.h"
#include "FXRbMenuCommand.h"
#include "FXRbToolbar.h"
#include "FXRbMenubar.h"
#include "FXRbOptionMenu.h"
#include "FXRbSwitcher.h"
#include "FXRbTabBar.h"
#include "FXRbTabBook.h"
#include "FXRbTabItem.h"
#include "FXRbScrollbar.h"
#include "FXRbScrollArea.h"
#include "FXRbScrollWindow.h"
#include "FXRbList.h"
#include "FXRbComboBox.h"
#include "FXRbListBox.h"
#include "FXRbTreeList.h"
#include "FXRbTreeListBox.h"
#include "FXRbImageView.h"
#include "FXRbTable.h"
#include "FXRbDragCorner.h"
#include "FXRbStatusbar.h"
#include "FXRbStatusline.h"
#include "FXRbMessageBox.h"
#include "FXRbDirList.h"
#include "FXRbSlider.h"
#include "FXRbSplitter.h"
#include "FXRb4Splitter.h"
#include "FXRbHeader.h"
#include "FXRbShutter.h"
#include "FXRbIconList.h"
#include "FXRbFileList.h"
#include "FXRbDirBox.h"
#include "FXRbDriveBox.h"
#include "FXRbDirSelector.h"
#include "FXRbDirDialog.h"
#include "FXRbFileSelector.h"
#include "FXRbFileDialog.h"
#include "FXRbColorSelector.h"
#include "FXRbColorDialog.h"
#include "FXRbFontSelector.h"
#include "FXRbFontDialog.h"
#include "FXRbText.h"
#include "FXRbDataTarget.h"
#include "FXRbProgressBar.h"
#include "FXRbSearchDialog.h"
#include "FXRbReplaceDialog.h"
#include "FXRbInputDialog.h"
#include "FXRbProgressDialog.h"
#include "FXRbMDIButton.h"
#include "FXRbMDIClient.h"
#include "FXRbMDIChild.h"
#include "FXRbDocument.h"
#include "FXRbToolbarGrip.h"
#include "FXRbToolbarShell.h"
#include "FXRbToolbarTab.h"
#include "FXRbPrintDialog.h"
#include "FXRbDebugTarget.h"
#ifdef WITH_FXSCINTILLA
#include "FXRbScintilla.h"
#endif

// OpenGL support
#include "FXRbGLVisual.h"
#include "FXRbGLContext.h"
#include "FXRbGLCanvas.h"
#include "FXRbGLViewer.h"
#include "FXRbGLObject.h"
#include "FXRbGLShape.h"

// Convert from Ruby to FOX 
extern void Get_FXRectangle(VALUE val, FXRectangle*& ptr);
extern void Get_FXPoint(VALUE val, FXPoint*& ptr);
extern void Get_FXSegment(VALUE val, FXSegment*& ptr);
extern void Get_FXArc(VALUE val, FXArc*& ptr);
extern void Get_FXPrinter(VALUE val, FXPrinter*& ptr);
extern void Get_FXEvent(VALUE val, FXEvent*& ptr);
extern void Get_FXStreamHashEntry(VALUE val, FXStreamHashEntry*& ptr);
extern void Get_FXStream(VALUE val, FXStream*& ptr);
extern void Get_FXFileStream(VALUE val, FXFileStream*& ptr);
extern void Get_FXMemoryStream(VALUE val, FXMemoryStream*& ptr);
extern void Get_FXObject(VALUE val, FXObject*& ptr);
extern void Get_FXRbObject(VALUE val, FXRbObject*& ptr);
extern void Get_FXDelegator(VALUE val, FXDelegator*& ptr);
extern void Get_FXRbDelegator(VALUE val, FXRbDelegator*& ptr);
extern void Get_FXDict(VALUE val, FXDict*& ptr);
extern void Get_FXRbDict(VALUE val, FXRbDict*& ptr);
extern void Get_FXStringDict(VALUE val, FXStringDict*& ptr);
extern void Get_FXRbStringDict(VALUE val, FXRbStringDict*& ptr);
extern void Get_FXRegistry(VALUE val, FXRegistry*& ptr);
extern void Get_FXAccelTable(VALUE val, FXAccelTable*& ptr);
extern void Get_FXRbAccelTable(VALUE val, FXRbAccelTable*& ptr);
extern void Get_FXRecentFiles(VALUE val, FXRecentFiles*& ptr);
extern void Get_FXRbRecentFiles(VALUE val, FXRbRecentFiles*& ptr);
extern void Get_FXCharset(VALUE val, FXCharset*& ptr);
extern void Get_FXApp(VALUE val, FXApp*& ptr);
extern void Get_FXRbApp(VALUE val, FXRbApp*& ptr);
extern void Get_FXId(VALUE val, FXId*& ptr);
extern void Get_FXRbId(VALUE val, FXRbId*& ptr);
extern void Get_FXVisual(VALUE val, FXVisual*& ptr);
extern void Get_FXRbVisual(VALUE val, FXRbVisual*& ptr);
extern void Get_FXFontDesc(VALUE val, FXFontDesc*& ptr);
extern void Get_FXFont(VALUE val, FXFont*& ptr);
extern void Get_FXRbFont(VALUE val, FXRbFont*& ptr);
extern void Get_FXCursor(VALUE val, FXCursor*& ptr);
extern void Get_FXRbCursor(VALUE val, FXRbCursor*& ptr);
extern void Get_FXGIFCursor(VALUE val, FXGIFCursor*& ptr);
extern void Get_FXRbGIFCursor(VALUE val, FXRbGIFCursor*& ptr);
extern void Get_FXDrawable(VALUE val, FXDrawable*& ptr);
extern void Get_FXRbDrawable(VALUE val, FXRbDrawable*& ptr);
extern void Get_FXBitmap(VALUE val, FXBitmap*& ptr);
extern void Get_FXRbBitmap(VALUE val, FXRbBitmap*& ptr);
extern void Get_FXRegion(VALUE val, FXRegion*& ptr);
extern void Get_FXDC(VALUE val, FXDC*& ptr);
extern void Get_FXRbDC(VALUE val, FXRbDC*& ptr);
extern void Get_FXDCWindow(VALUE val, FXDCWindow*& ptr);
extern void Get_FXRbDCWindow(VALUE val, FXRbDCWindow*& ptr);
extern void Get_FXPSBounds(VALUE val, FXPSBounds*& ptr);
extern void Get_FXDCPrint(VALUE val, FXDCPrint*& ptr);
extern void Get_FXRbDCPrint(VALUE val, FXRbDCPrint*& ptr);
extern void Get_FXFileAssoc(VALUE val, FXFileAssoc*& ptr);
extern void Get_FXIconDict(VALUE val, FXIconDict*& ptr);
extern void Get_FXRbIconDict(VALUE val, FXRbIconDict*& ptr);
extern void Get_FXFileDict(VALUE val, FXFileDict*& ptr);
extern void Get_FXRbFileDict(VALUE val, FXRbFileDict*& ptr);
extern void Get_FXWindow(VALUE val, FXWindow*& ptr);
extern void Get_FXRbWindow(VALUE val, FXRbWindow*& ptr);
extern void Get_FXFrame(VALUE val, FXFrame*& ptr);
extern void Get_FXRbFrame(VALUE val, FXRbFrame*& ptr);
extern void Get_FXHorizontalSeparator(VALUE val, FXHorizontalSeparator*& ptr);
extern void Get_FXRbHorizontalSeparator(VALUE val, FXRbHorizontalSeparator*& ptr);
extern void Get_FXVerticalSeparator(VALUE val, FXVerticalSeparator*& ptr);
extern void Get_FXRbVerticalSeparator(VALUE val, FXRbVerticalSeparator*& ptr);
extern void Get_FXLabel(VALUE val, FXLabel*& ptr);
extern void Get_FXRbLabel(VALUE val, FXRbLabel*& ptr);
extern void Get_FXDial(VALUE val, FXDial*& ptr);
extern void Get_FXRbDial(VALUE val, FXRbDial*& ptr);
extern void Get_FXColorWell(VALUE val, FXColorWell*& ptr);
extern void Get_FXRbColorWell(VALUE val, FXRbColorWell*& ptr);
extern void Get_FXTextField(VALUE val, FXTextField*& ptr);
extern void Get_FXRbTextField(VALUE val, FXRbTextField*& ptr);
extern void Get_FXButton(VALUE val, FXButton*& ptr);
extern void Get_FXRbButton(VALUE val, FXRbButton*& ptr);
extern void Get_FXToggleButton(VALUE val, FXToggleButton*& ptr);
extern void Get_FXRbToggleButton(VALUE val, FXRbToggleButton*& ptr);
extern void Get_FXCheckButton(VALUE val, FXCheckButton*& ptr);
extern void Get_FXRbCheckButton(VALUE val, FXRbCheckButton*& ptr);
extern void Get_FXRadioButton(VALUE val, FXRadioButton*& ptr);
extern void Get_FXRbRadioButton(VALUE val, FXRbRadioButton*& ptr);
extern void Get_FXArrowButton(VALUE val, FXArrowButton*& ptr);
extern void Get_FXRbArrowButton(VALUE val, FXRbArrowButton*& ptr);
extern void Get_FXMenuButton(VALUE val, FXMenuButton*& ptr);
extern void Get_FXRbMenuButton(VALUE val, FXRbMenuButton*& ptr);
extern void Get_FXComposite(VALUE val, FXComposite*& ptr);
extern void Get_FXRbComposite(VALUE val, FXRbComposite*& ptr);
extern void Get_FXSpinner(VALUE val, FXSpinner*& ptr);
extern void Get_FXRbSpinner(VALUE val, FXRbSpinner*& ptr);
extern void Get_FXRootWindow(VALUE val, FXRootWindow*& ptr);
extern void Get_FXRbRootWindow(VALUE val, FXRbRootWindow*& ptr);
extern void Get_FXCanvas(VALUE val, FXCanvas*& ptr);
extern void Get_FXRbCanvas(VALUE val, FXRbCanvas*& ptr);
extern void Get_FXShell(VALUE val, FXShell*& ptr);
extern void Get_FXRbShell(VALUE val, FXRbShell*& ptr);
extern void Get_FXTooltip(VALUE val, FXTooltip*& ptr);
extern void Get_FXRbTooltip(VALUE val, FXRbTooltip*& ptr);
extern void Get_FXPopup(VALUE val, FXPopup*& ptr);
extern void Get_FXRbPopup(VALUE val, FXRbPopup*& ptr);
extern void Get_FXTopWindow(VALUE val, FXTopWindow*& ptr);
extern void Get_FXRbTopWindow(VALUE val, FXRbTopWindow*& ptr);
extern void Get_FXDialogBox(VALUE val, FXDialogBox*& ptr);
extern void Get_FXRbDialogBox(VALUE val, FXRbDialogBox*& ptr);
extern void Get_FXMainWindow(VALUE val, FXMainWindow*& ptr);
extern void Get_FXRbMainWindow(VALUE val, FXRbMainWindow*& ptr);
extern void Get_FXMenuPane(VALUE val, FXMenuPane*& ptr);
extern void Get_FXRbMenuPane(VALUE val, FXRbMenuPane*& ptr);
extern void Get_FXMenuCaption(VALUE val, FXMenuCaption*& ptr);
extern void Get_FXRbMenuCaption(VALUE val, FXRbMenuCaption*& ptr);
extern void Get_FXMenuSeparator(VALUE val, FXMenuSeparator*& ptr);
extern void Get_FXRbMenuSeparator(VALUE val, FXRbMenuSeparator*& ptr);
extern void Get_FXMenuTitle(VALUE val, FXMenuTitle*& ptr);
extern void Get_FXRbMenuTitle(VALUE val, FXRbMenuTitle*& ptr);
extern void Get_FXMenuCascade(VALUE val, FXMenuCascade*& ptr);
extern void Get_FXRbMenuCascade(VALUE val, FXRbMenuCascade*& ptr);
extern void Get_FXMenuCommand(VALUE val, FXMenuCommand*& ptr);
extern void Get_FXRbMenuCommand(VALUE val, FXRbMenuCommand*& ptr);
extern void Get_FXToolbar(VALUE val, FXToolbar*& ptr);
extern void Get_FXRbToolbar(VALUE val, FXRbToolbar*& ptr);
extern void Get_FXMenubar(VALUE val, FXMenubar*& ptr);
extern void Get_FXRbMenubar(VALUE val, FXRbMenubar*& ptr);
extern void Get_FXOption(VALUE val, FXOption*& ptr);
extern void Get_FXRbOption(VALUE val, FXRbOption*& ptr);
extern void Get_FXOptionMenu(VALUE val, FXOptionMenu*& ptr);
extern void Get_FXRbOptionMenu(VALUE val, FXRbOptionMenu*& ptr);
extern void Get_FXTabItem(VALUE val, FXTabItem*& ptr);
extern void Get_FXRbTabItem(VALUE val, FXRbTabItem*& ptr);
extern void Get_FXTabBar(VALUE val, FXTabBar*& ptr);
extern void Get_FXRbTabBar(VALUE val, FXRbTabBar*& ptr);
extern void Get_FXTabBook(VALUE val, FXTabBook*& ptr);
extern void Get_FXRbTabBook(VALUE val, FXRbTabBook*& ptr);
extern void Get_FXScrollbar(VALUE val, FXScrollbar*& ptr);
extern void Get_FXRbScrollbar(VALUE val, FXRbScrollbar*& ptr);
extern void Get_FXScrollCorner(VALUE val, FXScrollCorner*& ptr);
extern void Get_FXRbScrollCorner(VALUE val, FXRbScrollCorner*& ptr);
extern void Get_FXScrollArea(VALUE val, FXScrollArea*& ptr);
extern void Get_FXRbScrollArea(VALUE val, FXRbScrollArea*& ptr);
extern void Get_FXScrollWindow(VALUE val, FXScrollWindow*& ptr);
extern void Get_FXRbScrollWindow(VALUE val, FXRbScrollWindow*& ptr);
extern void Get_FXListItem(VALUE val, FXListItem*& ptr);
extern void Get_FXRbListItem(VALUE val, FXRbListItem*& ptr);
extern void Get_FXList(VALUE val, FXList*& ptr);
extern void Get_FXRbList(VALUE val, FXRbList*& ptr);
extern void Get_FXComboBox(VALUE val, FXComboBox*& ptr);
extern void Get_FXRbComboBox(VALUE val, FXRbComboBox*& ptr);
extern void Get_FXListBox(VALUE val, FXListBox*& ptr);
extern void Get_FXRbListBox(VALUE val, FXRbListBox*& ptr);
extern void Get_FXTreeItem(VALUE val, FXTreeItem*& ptr);
extern void Get_FXRbTreeItem(VALUE val, FXRbTreeItem*& ptr);
extern void Get_FXTreeList(VALUE val, FXTreeList*& ptr);
extern void Get_FXRbTreeList(VALUE val, FXRbTreeList*& ptr);
extern void Get_FXTreeListBox(VALUE val, FXTreeListBox*& ptr);
extern void Get_FXRbTreeListBox(VALUE val, FXRbTreeListBox*& ptr);
extern void Get_FXImageView(VALUE val, FXImageView*& ptr);
extern void Get_FXRbImageView(VALUE val, FXRbImageView*& ptr);
extern void Get_FXTablePos(VALUE val, FXTablePos*& ptr);
extern void Get_FXTableRange(VALUE val, FXTableRange*& ptr);
extern void Get_FXTableItem(VALUE val, FXTableItem*& ptr);
extern void Get_FXRbTableItem(VALUE val, FXRbTableItem*& ptr);
extern void Get_FXTable(VALUE val, FXTable*& ptr);
extern void Get_FXRbTable(VALUE val, FXRbTable*& ptr);
extern void Get_FXDragCorner(VALUE val, FXDragCorner*& ptr);
extern void Get_FXRbDragCorner(VALUE val, FXRbDragCorner*& ptr);
extern void Get_FXStatusbar(VALUE val, FXStatusbar*& ptr);
extern void Get_FXRbStatusbar(VALUE val, FXRbStatusbar*& ptr);
extern void Get_FXStatusline(VALUE val, FXStatusline*& ptr);
extern void Get_FXRbStatusline(VALUE val, FXRbStatusline*& ptr);
extern void Get_FXMessageBox(VALUE val, FXMessageBox*& ptr);
extern void Get_FXRbMessageBox(VALUE val, FXRbMessageBox*& ptr);
extern void Get_FXDirItem(VALUE val, FXDirItem*& ptr);
extern void Get_FXRbDirItem(VALUE val, FXRbDirItem*& ptr);
extern void Get_FXDirList(VALUE val, FXDirList*& ptr);
extern void Get_FXRbDirList(VALUE val, FXRbDirList*& ptr);
extern void Get_FXSlider(VALUE val, FXSlider*& ptr);
extern void Get_FXRbSlider(VALUE val, FXRbSlider*& ptr);
extern void Get_FXHeaderItem(VALUE val, FXHeaderItem*& ptr);
extern void Get_FXRbHeaderItem(VALUE val, FXRbHeaderItem*& ptr);
extern void Get_FXHeader(VALUE val, FXHeader*& ptr);
extern void Get_FXRbHeader(VALUE val, FXRbHeader*& ptr);
extern void Get_FXShutterItem(VALUE val, FXShutterItem*& ptr);
extern void Get_FXRbShutterItem(VALUE val, FXRbShutterItem*& ptr);
extern void Get_FXShutter(VALUE val, FXShutter*& ptr);
extern void Get_FXRbShutter(VALUE val, FXRbShutter*& ptr);
extern void Get_FXIconItem(VALUE val, FXIconItem*& ptr);
extern void Get_FXRbIconItem(VALUE val, FXRbIconItem*& ptr);
extern void Get_FXIconList(VALUE val, FXIconList*& ptr);
extern void Get_FXRbIconList(VALUE val, FXRbIconList*& ptr);
extern void Get_FXFileItem(VALUE val, FXFileItem*& ptr);
extern void Get_FXRbFileItem(VALUE val, FXRbFileItem*& ptr);
extern void Get_FXFileList(VALUE val, FXFileList*& ptr);
extern void Get_FXRbFileList(VALUE val, FXRbFileList*& ptr);
extern void Get_FXDirBox(VALUE val, FXDirBox*& ptr);
extern void Get_FXRbDirBox(VALUE val, FXRbDirBox*& ptr);
extern void Get_FXDriveBox(VALUE val, FXDriveBox*& ptr);
extern void Get_FXRbDriveBox(VALUE val, FXRbDriveBox*& ptr);
extern void Get_FXDirSelector(VALUE val, FXDirSelector*& ptr);
extern void Get_FXRbDirSelector(VALUE val, FXRbDirSelector*& ptr);
extern void Get_FXFileSelector(VALUE val, FXFileSelector*& ptr);
extern void Get_FXRbFileSelector(VALUE val, FXRbFileSelector*& ptr);
extern void Get_FXColorSelector(VALUE val, FXColorSelector*& ptr);
extern void Get_FXRbColorSelector(VALUE val, FXRbColorSelector*& ptr);
extern void Get_FXFontSelector(VALUE val, FXFontSelector*& ptr);
extern void Get_FXRbFontSelector(VALUE val, FXRbFontSelector*& ptr);
extern void Get_FXText(VALUE val, FXText*& ptr);
extern void Get_FXRbText(VALUE val, FXRbText*& ptr);
extern void Get_FXDataTarget(VALUE val, FXDataTarget*& ptr);
extern void Get_FXRbDataTarget(VALUE val, FXRbDataTarget*& ptr);
extern void Get_FXProgressBar(VALUE val, FXProgressBar*& ptr);
extern void Get_FXRbProgressBar(VALUE val, FXRbProgressBar*& ptr);
extern void Get_FXToolbarGrip(VALUE val, FXToolbarGrip*& ptr);
extern void Get_FXRbToolbarGrip(VALUE val, FXRbToolbarGrip*& ptr);
extern void Get_FXToolbarShell(VALUE val, FXToolbarShell*& ptr);
extern void Get_FXRbToolbarShell(VALUE val, FXRbToolbarShell*& ptr);
extern void Get_FXToolbarTab(VALUE val, FXToolbarTab*& ptr);
extern void Get_FXRbToolbarTab(VALUE val, FXRbToolbarTab*& ptr);
extern void Get_FXDebugTarget(VALUE val, FXDebugTarget*& ptr);
extern void Get_FXRbDebugTarget(VALUE val, FXRbDebugTarget*& ptr);
extern void Get_FXDirDialog(VALUE val, FXDirDialog*& ptr);
extern void Get_FXRbDirDialog(VALUE val, FXRbDirDialog*& ptr);
extern void Get_FXFileDialog(VALUE val, FXFileDialog*& ptr);
extern void Get_FXRbFileDialog(VALUE val, FXRbFileDialog*& ptr);
extern void Get_FXColorDialog(VALUE val, FXColorDialog*& ptr);
extern void Get_FXRbColorDialog(VALUE val, FXRbColorDialog*& ptr);
extern void Get_FXFontDialog(VALUE val, FXFontDialog*& ptr);
extern void Get_FXRbFontDialog(VALUE val, FXRbFontDialog*& ptr);
extern void Get_FXSearchDialog(VALUE val, FXSearchDialog*& ptr);
extern void Get_FXRbSearchDialog(VALUE val, FXRbSearchDialog*& ptr);
extern void Get_FXReplaceDialog(VALUE val, FXReplaceDialog*& ptr);
extern void Get_FXRbReplaceDialog(VALUE val, FXRbReplaceDialog*& ptr);
extern void Get_FXInputDialog(VALUE val, FXInputDialog*& ptr);
extern void Get_FXRbInputDialog(VALUE val, FXRbInputDialog*& ptr);
extern void Get_FXPrintDialog(VALUE val, FXPrintDialog*& ptr);
extern void Get_FXRbPrintDialog(VALUE val, FXRbPrintDialog*& ptr);
extern void Get_FXIcon(VALUE val, FXIcon*& ptr);
extern void Get_FXRbIcon(VALUE val, FXRbIcon*& ptr);
extern void Get_FXBMPIcon(VALUE val, FXBMPIcon*& ptr);
extern void Get_FXRbBMPIcon(VALUE val, FXRbBMPIcon*& ptr);
extern void Get_FXGIFIcon(VALUE val, FXGIFIcon*& ptr);
extern void Get_FXRbGIFIcon(VALUE val, FXRbGIFIcon*& ptr);
extern void Get_FXPNGIcon(VALUE val, FXPNGIcon*& ptr);
extern void Get_FXRbPNGIcon(VALUE val, FXRbPNGIcon*& ptr);
extern void Get_FXXPMIcon(VALUE val, FXXPMIcon*& ptr);
extern void Get_FXRbXPMIcon(VALUE val, FXRbXPMIcon*& ptr);
extern void Get_FXImage(VALUE val, FXImage*& ptr);
extern void Get_FXRbImage(VALUE val, FXRbImage*& ptr);
extern void Get_FXBMPImage(VALUE val, FXBMPImage*& ptr);
extern void Get_FXRbBMPImage(VALUE val, FXRbBMPImage*& ptr);
extern void Get_FXGIFImage(VALUE val, FXGIFImage*& ptr);
extern void Get_FXRbGIFImage(VALUE val, FXRbGIFImage*& ptr);
extern void Get_FXPNGImage(VALUE val, FXPNGImage*& ptr);
extern void Get_FXRbPNGImage(VALUE val, FXRbPNGImage*& ptr);
extern void Get_FXXPMImage(VALUE val, FXXPMImage*& ptr);
extern void Get_FXRbXPMImage(VALUE val, FXRbXPMImage*& ptr);
extern void Get_FXPacker(VALUE val, FXPacker*& ptr);
extern void Get_FXRbPacker(VALUE val, FXRbPacker*& ptr);
extern void Get_FXHorizontalFrame(VALUE val, FXHorizontalFrame*& ptr);
extern void Get_FXRbHorizontalFrame(VALUE val, FXRbHorizontalFrame*& ptr);
extern void Get_FXVerticalFrame(VALUE val, FXVerticalFrame*& ptr);
extern void Get_FXRbVerticalFrame(VALUE val, FXRbVerticalFrame*& ptr);
extern void Get_FXMatrix(VALUE val, FXMatrix*& ptr);
extern void Get_FXRbMatrix(VALUE val, FXRbMatrix*& ptr);
extern void Get_FXGroupBox(VALUE val, FXGroupBox*& ptr);
extern void Get_FXRbGroupBox(VALUE val, FXRbGroupBox*& ptr);
extern void Get_FXSwitcher(VALUE val, FXSwitcher*& ptr);
extern void Get_FXRbSwitcher(VALUE val, FXRbSwitcher*& ptr);
extern void Get_FXSplitter(VALUE val, FXSplitter*& ptr);
extern void Get_FXRbSplitter(VALUE val, FXRbSplitter*& ptr);
extern void Get_FX4Splitter(VALUE val, FX4Splitter*& ptr);
extern void Get_FXRb4Splitter(VALUE val, FXRb4Splitter*& ptr);
extern void Get_FXMDIDeleteButton(VALUE val, FXMDIDeleteButton*& ptr);
extern void Get_FXRbMDIDeleteButton(VALUE val, FXRbMDIDeleteButton*& ptr);
extern void Get_FXMDIRestoreButton(VALUE val, FXMDIRestoreButton*& ptr);
extern void Get_FXRbMDIRestoreButton(VALUE val, FXRbMDIRestoreButton*& ptr);
extern void Get_FXMDIMaximizeButton(VALUE val, FXMDIMaximizeButton*& ptr);
extern void Get_FXRbMDIMaximizeButton(VALUE val, FXRbMDIMaximizeButton*& ptr);
extern void Get_FXMDIMinimizeButton(VALUE val, FXMDIMinimizeButton*& ptr);
extern void Get_FXRbMDIMinimizeButton(VALUE val, FXRbMDIMinimizeButton*& ptr);
extern void Get_FXMDIWindowButton(VALUE val, FXMDIWindowButton*& ptr);
extern void Get_FXRbMDIWindowButton(VALUE val, FXRbMDIWindowButton*& ptr);
extern void Get_FXMDIMenu(VALUE val, FXMDIMenu*& ptr);
extern void Get_FXRbMDIMenu(VALUE val, FXRbMDIMenu*& ptr);
extern void Get_FXMDIClient(VALUE val, FXMDIClient*& ptr);
extern void Get_FXRbMDIClient(VALUE val, FXRbMDIClient*& ptr);
extern void Get_FXMDIChild(VALUE val, FXMDIChild*& ptr);
extern void Get_FXRbMDIChild(VALUE val, FXRbMDIChild*& ptr);
extern void Get_FXDocument(VALUE val, FXDocument*& ptr);
extern void Get_FXRbDocument(VALUE val, FXRbDocument*& ptr);
extern void Get_FXRectangle(VALUE val, FXRectangle*& ptr);
extern void Get_FXPoint(VALUE val, FXPoint*& ptr);
extern void Get_FXSegment(VALUE val, FXSegment*& ptr);
extern void Get_FXArc(VALUE val, FXArc*& ptr);
extern void Get_FXPrinter(VALUE val, FXPrinter*& ptr);
extern void Get_FXEvent(VALUE val, FXEvent*& ptr);
extern void Get_FXVec(VALUE val, FXVec*& ptr);
extern void Get_FXHVec(VALUE val, FXHVec*& ptr);
extern void Get_FXQuat(VALUE val, FXQuat*& ptr);
extern void Get_FXHMat(VALUE val, FXHMat*& ptr);
extern void Get_FXRange(VALUE val, FXRange*& ptr);
extern void Get_FXDVec(VALUE val, FXDVec*& ptr);
extern void Get_FXDHVec(VALUE val, FXDHVec*& ptr);
extern void Get_FXDQuat(VALUE val, FXDQuat*& ptr);
extern void Get_FXDHMat(VALUE val, FXDHMat*& ptr);
extern void Get_FXGLVisual(VALUE val, FXGLVisual*& ptr);
extern void Get_FXRbGLVisual(VALUE val, FXRbGLVisual*& ptr);
extern void Get_FXGLContext(VALUE val, FXGLContext*& ptr);
extern void Get_FXRbGLContext(VALUE val, FXRbGLContext*& ptr);
extern void Get_FXGLCanvas(VALUE val, FXGLCanvas*& ptr);
extern void Get_FXRbGLCanvas(VALUE val, FXRbGLCanvas*& ptr);
extern void Get_FXViewport(VALUE val, FXViewport*& ptr);
extern void Get_FXLight(VALUE val, FXLight*& ptr);
extern void Get_FXMaterial(VALUE val, FXMaterial*& ptr);
extern void Get_FXGLViewer(VALUE val, FXGLViewer*& ptr);
extern void Get_FXRbGLViewer(VALUE val, FXRbGLViewer*& ptr);
extern void Get_FXGLObject(VALUE val, FXGLObject*& ptr);
extern void Get_FXRbGLObject(VALUE val, FXRbGLObject*& ptr);
extern void Get_FXGLGroup(VALUE val, FXGLGroup*& ptr);
extern void Get_FXRbGLGroup(VALUE val, FXRbGLGroup*& ptr);
extern void Get_FXGLShape(VALUE val, FXGLShape*& ptr);
extern void Get_FXRbGLShape(VALUE val, FXRbGLShape*& ptr);
extern void Get_FXPicker(VALUE val, FXPicker*& ptr);
extern void Get_FXRbPicker(VALUE val, FXRbPicker*& ptr);
extern void Get_FXColorBar(VALUE val, FXColorBar*& ptr);
extern void Get_FXRbColorBar(VALUE val, FXRbColorBar*& ptr);
extern void Get_FXColorWheel(VALUE val, FXColorWheel*& ptr);
extern void Get_FXRbColorWheel(VALUE val, FXRbColorWheel*& ptr);
extern void Get_FXICOIcon(VALUE val, FXICOIcon*& ptr);
extern void Get_FXRbICOIcon(VALUE val, FXRbICOIcon*& ptr);
extern void Get_FXICOImage(VALUE val, FXICOImage*& ptr);
extern void Get_FXRbICOImage(VALUE val, FXRbICOImage*& ptr);
extern void Get_FXTGAImage(VALUE val, FXTGAImage*& ptr);
extern void Get_FXRbTGAImage(VALUE val, FXRbTGAImage*& ptr);
extern void Get_FXTGAIcon(VALUE val, FXTGAIcon*& ptr);
extern void Get_FXRbTGAIcon(VALUE val, FXRbTGAIcon*& ptr);
#ifdef WITH_FXSCINTILLA
extern void Get_FXScintilla(VALUE val, FXScintilla*& ptr);
extern void Get_FXRbScintilla(VALUE val, FXRbScintilla*& ptr);
#endif

extern VALUE Wrap_FXRectangle(VALUE klass, FXRectangle* ptr, int owned);
extern VALUE Wrap_FXPoint(VALUE klass, FXPoint* ptr, int owned);
extern VALUE Wrap_FXSegment(VALUE klass, FXSegment* ptr, int owned);
extern VALUE Wrap_FXArc(VALUE klass, FXArc* ptr, int owned);
extern VALUE Wrap_FXPrinter(VALUE klass, FXPrinter* ptr, int owned);
extern VALUE Wrap_FXEvent(VALUE klass, FXEvent* ptr, int owned);
extern VALUE Wrap_FXStreamHashEntry(VALUE klass, FXStreamHashEntry* ptr, int owned);
extern VALUE Wrap_FXStream(VALUE klass, FXStream* ptr, int owned);
extern VALUE Wrap_FXFileStream(VALUE klass, FXFileStream* ptr, int owned);
extern VALUE Wrap_FXMemoryStream(VALUE klass, FXMemoryStream* ptr, int owned);
extern VALUE Wrap_FXObject(VALUE klass, FXObject* ptr, int owned);
extern VALUE Wrap_FXRbObject(VALUE klass, FXRbObject* ptr, int owned);
extern VALUE Wrap_FXDelegator(VALUE klass, FXDelegator* ptr, int owned);
extern VALUE Wrap_FXRbDelegator(VALUE klass, FXRbDelegator* ptr, int owned);
extern VALUE Wrap_FXDict(VALUE klass, FXDict* ptr, int owned);
extern VALUE Wrap_FXRbDict(VALUE klass, FXRbDict* ptr, int owned);
extern VALUE Wrap_FXStringDict(VALUE klass, FXStringDict* ptr, int owned);
extern VALUE Wrap_FXRbStringDict(VALUE klass, FXRbStringDict* ptr, int owned);
extern VALUE Wrap_FXRegistry(VALUE klass, FXRegistry* ptr, int owned);
extern VALUE Wrap_FXAccelTable(VALUE klass, FXAccelTable* ptr, int owned);
extern VALUE Wrap_FXRbAccelTable(VALUE klass, FXRbAccelTable* ptr, int owned);
extern VALUE Wrap_FXRecentFiles(VALUE klass, FXRecentFiles* ptr, int owned);
extern VALUE Wrap_FXRbRecentFiles(VALUE klass, FXRbRecentFiles* ptr, int owned);
extern VALUE Wrap_FXCharset(VALUE klass, FXCharset* ptr, int owned);
extern VALUE Wrap_FXApp(VALUE klass, FXApp* ptr, int owned);
extern VALUE Wrap_FXRbApp(VALUE klass, FXRbApp* ptr, int owned);
extern VALUE Wrap_FXId(VALUE klass, FXId* ptr, int owned);
extern VALUE Wrap_FXRbId(VALUE klass, FXRbId* ptr, int owned);
extern VALUE Wrap_FXVisual(VALUE klass, FXVisual* ptr, int owned);
extern VALUE Wrap_FXRbVisual(VALUE klass, FXRbVisual* ptr, int owned);
extern VALUE Wrap_FXFontDesc(VALUE klass, FXFontDesc* ptr, int owned);
extern VALUE Wrap_FXFont(VALUE klass, FXFont* ptr, int owned);
extern VALUE Wrap_FXRbFont(VALUE klass, FXRbFont* ptr, int owned);
extern VALUE Wrap_FXCursor(VALUE klass, FXCursor* ptr, int owned);
extern VALUE Wrap_FXRbCursor(VALUE klass, FXRbCursor* ptr, int owned);
extern VALUE Wrap_FXGIFCursor(VALUE klass, FXGIFCursor* ptr, int owned);
extern VALUE Wrap_FXRbGIFCursor(VALUE klass, FXRbGIFCursor* ptr, int owned);
extern VALUE Wrap_FXDrawable(VALUE klass, FXDrawable* ptr, int owned);
extern VALUE Wrap_FXRbDrawable(VALUE klass, FXRbDrawable* ptr, int owned);
extern VALUE Wrap_FXBitmap(VALUE klass, FXBitmap* ptr, int owned);
extern VALUE Wrap_FXRbBitmap(VALUE klass, FXRbBitmap* ptr, int owned);
extern VALUE Wrap_FXRegion(VALUE klass, FXRegion* ptr, int owned);
extern VALUE Wrap_FXDC(VALUE klass, FXDC* ptr, int owned);
extern VALUE Wrap_FXRbDC(VALUE klass, FXRbDC* ptr, int owned);
extern VALUE Wrap_FXDCWindow(VALUE klass, FXDCWindow* ptr, int owned);
extern VALUE Wrap_FXRbDCWindow(VALUE klass, FXRbDCWindow* ptr, int owned);
extern VALUE Wrap_FXPSBounds(VALUE klass, FXPSBounds* ptr, int owned);
extern VALUE Wrap_FXDCPrint(VALUE klass, FXDCPrint* ptr, int owned);
extern VALUE Wrap_FXRbDCPrint(VALUE klass, FXRbDCPrint* ptr, int owned);
extern VALUE Wrap_FXFileAssoc(VALUE klass, FXFileAssoc* ptr, int owned);
extern VALUE Wrap_FXIconDict(VALUE klass, FXIconDict* ptr, int owned);
extern VALUE Wrap_FXRbIconDict(VALUE klass, FXRbIconDict* ptr, int owned);
extern VALUE Wrap_FXFileDict(VALUE klass, FXFileDict* ptr, int owned);
extern VALUE Wrap_FXRbFileDict(VALUE klass, FXRbFileDict* ptr, int owned);
extern VALUE Wrap_FXWindow(VALUE klass, FXWindow* ptr, int owned);
extern VALUE Wrap_FXRbWindow(VALUE klass, FXRbWindow* ptr, int owned);
extern VALUE Wrap_FXFrame(VALUE klass, FXFrame* ptr, int owned);
extern VALUE Wrap_FXRbFrame(VALUE klass, FXRbFrame* ptr, int owned);
extern VALUE Wrap_FXHorizontalSeparator(VALUE klass, FXHorizontalSeparator* ptr, int owned);
extern VALUE Wrap_FXRbHorizontalSeparator(VALUE klass, FXRbHorizontalSeparator* ptr, int owned);
extern VALUE Wrap_FXVerticalSeparator(VALUE klass, FXVerticalSeparator* ptr, int owned);
extern VALUE Wrap_FXRbVerticalSeparator(VALUE klass, FXRbVerticalSeparator* ptr, int owned);
extern VALUE Wrap_FXLabel(VALUE klass, FXLabel* ptr, int owned);
extern VALUE Wrap_FXRbLabel(VALUE klass, FXRbLabel* ptr, int owned);
extern VALUE Wrap_FXDial(VALUE klass, FXDial* ptr, int owned);
extern VALUE Wrap_FXRbDial(VALUE klass, FXRbDial* ptr, int owned);
extern VALUE Wrap_FXColorWell(VALUE klass, FXColorWell* ptr, int owned);
extern VALUE Wrap_FXRbColorWell(VALUE klass, FXRbColorWell* ptr, int owned);
extern VALUE Wrap_FXTextField(VALUE klass, FXTextField* ptr, int owned);
extern VALUE Wrap_FXRbTextField(VALUE klass, FXRbTextField* ptr, int owned);
extern VALUE Wrap_FXButton(VALUE klass, FXButton* ptr, int owned);
extern VALUE Wrap_FXRbButton(VALUE klass, FXRbButton* ptr, int owned);
extern VALUE Wrap_FXToggleButton(VALUE klass, FXToggleButton* ptr, int owned);
extern VALUE Wrap_FXRbToggleButton(VALUE klass, FXRbToggleButton* ptr, int owned);
extern VALUE Wrap_FXCheckButton(VALUE klass, FXCheckButton* ptr, int owned);
extern VALUE Wrap_FXRbCheckButton(VALUE klass, FXRbCheckButton* ptr, int owned);
extern VALUE Wrap_FXRadioButton(VALUE klass, FXRadioButton* ptr, int owned);
extern VALUE Wrap_FXRbRadioButton(VALUE klass, FXRbRadioButton* ptr, int owned);
extern VALUE Wrap_FXArrowButton(VALUE klass, FXArrowButton* ptr, int owned);
extern VALUE Wrap_FXRbArrowButton(VALUE klass, FXRbArrowButton* ptr, int owned);
extern VALUE Wrap_FXMenuButton(VALUE klass, FXMenuButton* ptr, int owned);
extern VALUE Wrap_FXRbMenuButton(VALUE klass, FXRbMenuButton* ptr, int owned);
extern VALUE Wrap_FXComposite(VALUE klass, FXComposite* ptr, int owned);
extern VALUE Wrap_FXRbComposite(VALUE klass, FXRbComposite* ptr, int owned);
extern VALUE Wrap_FXSpinner(VALUE klass, FXSpinner* ptr, int owned);
extern VALUE Wrap_FXRbSpinner(VALUE klass, FXRbSpinner* ptr, int owned);
extern VALUE Wrap_FXRootWindow(VALUE klass, FXRootWindow* ptr, int owned);
extern VALUE Wrap_FXRbRootWindow(VALUE klass, FXRbRootWindow* ptr, int owned);
extern VALUE Wrap_FXCanvas(VALUE klass, FXCanvas* ptr, int owned);
extern VALUE Wrap_FXRbCanvas(VALUE klass, FXRbCanvas* ptr, int owned);
extern VALUE Wrap_FXShell(VALUE klass, FXShell* ptr, int owned);
extern VALUE Wrap_FXRbShell(VALUE klass, FXRbShell* ptr, int owned);
extern VALUE Wrap_FXTooltip(VALUE klass, FXTooltip* ptr, int owned);
extern VALUE Wrap_FXRbTooltip(VALUE klass, FXRbTooltip* ptr, int owned);
extern VALUE Wrap_FXPopup(VALUE klass, FXPopup* ptr, int owned);
extern VALUE Wrap_FXRbPopup(VALUE klass, FXRbPopup* ptr, int owned);
extern VALUE Wrap_FXTopWindow(VALUE klass, FXTopWindow* ptr, int owned);
extern VALUE Wrap_FXRbTopWindow(VALUE klass, FXRbTopWindow* ptr, int owned);
extern VALUE Wrap_FXDialogBox(VALUE klass, FXDialogBox* ptr, int owned);
extern VALUE Wrap_FXRbDialogBox(VALUE klass, FXRbDialogBox* ptr, int owned);
extern VALUE Wrap_FXMainWindow(VALUE klass, FXMainWindow* ptr, int owned);
extern VALUE Wrap_FXRbMainWindow(VALUE klass, FXRbMainWindow* ptr, int owned);
extern VALUE Wrap_FXMenuPane(VALUE klass, FXMenuPane* ptr, int owned);
extern VALUE Wrap_FXRbMenuPane(VALUE klass, FXRbMenuPane* ptr, int owned);
extern VALUE Wrap_FXMenuCaption(VALUE klass, FXMenuCaption* ptr, int owned);
extern VALUE Wrap_FXRbMenuCaption(VALUE klass, FXRbMenuCaption* ptr, int owned);
extern VALUE Wrap_FXMenuSeparator(VALUE klass, FXMenuSeparator* ptr, int owned);
extern VALUE Wrap_FXRbMenuSeparator(VALUE klass, FXRbMenuSeparator* ptr, int owned);
extern VALUE Wrap_FXMenuTitle(VALUE klass, FXMenuTitle* ptr, int owned);
extern VALUE Wrap_FXRbMenuTitle(VALUE klass, FXRbMenuTitle* ptr, int owned);
extern VALUE Wrap_FXMenuCascade(VALUE klass, FXMenuCascade* ptr, int owned);
extern VALUE Wrap_FXRbMenuCascade(VALUE klass, FXRbMenuCascade* ptr, int owned);
extern VALUE Wrap_FXMenuCommand(VALUE klass, FXMenuCommand* ptr, int owned);
extern VALUE Wrap_FXRbMenuCommand(VALUE klass, FXRbMenuCommand* ptr, int owned);
extern VALUE Wrap_FXToolbar(VALUE klass, FXToolbar* ptr, int owned);
extern VALUE Wrap_FXRbToolbar(VALUE klass, FXRbToolbar* ptr, int owned);
extern VALUE Wrap_FXMenubar(VALUE klass, FXMenubar* ptr, int owned);
extern VALUE Wrap_FXRbMenubar(VALUE klass, FXRbMenubar* ptr, int owned);
extern VALUE Wrap_FXOption(VALUE klass, FXOption* ptr, int owned);
extern VALUE Wrap_FXRbOption(VALUE klass, FXRbOption* ptr, int owned);
extern VALUE Wrap_FXOptionMenu(VALUE klass, FXOptionMenu* ptr, int owned);
extern VALUE Wrap_FXRbOptionMenu(VALUE klass, FXRbOptionMenu* ptr, int owned);
extern VALUE Wrap_FXTabItem(VALUE klass, FXTabItem* ptr, int owned);
extern VALUE Wrap_FXRbTabItem(VALUE klass, FXRbTabItem* ptr, int owned);
extern VALUE Wrap_FXTabBar(VALUE klass, FXTabBar* ptr, int owned);
extern VALUE Wrap_FXRbTabBar(VALUE klass, FXRbTabBar* ptr, int owned);
extern VALUE Wrap_FXTabBook(VALUE klass, FXTabBook* ptr, int owned);
extern VALUE Wrap_FXRbTabBook(VALUE klass, FXRbTabBook* ptr, int owned);
extern VALUE Wrap_FXScrollbar(VALUE klass, FXScrollbar* ptr, int owned);
extern VALUE Wrap_FXRbScrollbar(VALUE klass, FXRbScrollbar* ptr, int owned);
extern VALUE Wrap_FXScrollCorner(VALUE klass, FXScrollCorner* ptr, int owned);
extern VALUE Wrap_FXRbScrollCorner(VALUE klass, FXRbScrollCorner* ptr, int owned);
extern VALUE Wrap_FXScrollArea(VALUE klass, FXScrollArea* ptr, int owned);
extern VALUE Wrap_FXRbScrollArea(VALUE klass, FXRbScrollArea* ptr, int owned);
extern VALUE Wrap_FXScrollWindow(VALUE klass, FXScrollWindow* ptr, int owned);
extern VALUE Wrap_FXRbScrollWindow(VALUE klass, FXRbScrollWindow* ptr, int owned);
extern VALUE Wrap_FXListItem(VALUE klass, FXListItem* ptr, int owned);
extern VALUE Wrap_FXRbListItem(VALUE klass, FXRbListItem* ptr, int owned);
extern VALUE Wrap_FXList(VALUE klass, FXList* ptr, int owned);
extern VALUE Wrap_FXRbList(VALUE klass, FXRbList* ptr, int owned);
extern VALUE Wrap_FXComboBox(VALUE klass, FXComboBox* ptr, int owned);
extern VALUE Wrap_FXRbComboBox(VALUE klass, FXRbComboBox* ptr, int owned);
extern VALUE Wrap_FXListBox(VALUE klass, FXListBox* ptr, int owned);
extern VALUE Wrap_FXRbListBox(VALUE klass, FXRbListBox* ptr, int owned);
extern VALUE Wrap_FXTreeItem(VALUE klass, FXTreeItem* ptr, int owned);
extern VALUE Wrap_FXRbTreeItem(VALUE klass, FXRbTreeItem* ptr, int owned);
extern VALUE Wrap_FXTreeList(VALUE klass, FXTreeList* ptr, int owned);
extern VALUE Wrap_FXRbTreeList(VALUE klass, FXRbTreeList* ptr, int owned);
extern VALUE Wrap_FXTreeListBox(VALUE klass, FXTreeListBox* ptr, int owned);
extern VALUE Wrap_FXRbTreeListBox(VALUE klass, FXRbTreeListBox* ptr, int owned);
extern VALUE Wrap_FXImageView(VALUE klass, FXImageView* ptr, int owned);
extern VALUE Wrap_FXRbImageView(VALUE klass, FXRbImageView* ptr, int owned);
extern VALUE Wrap_FXTablePos(VALUE klass, FXTablePos* ptr, int owned);
extern VALUE Wrap_FXTableRange(VALUE klass, FXTableRange* ptr, int owned);
extern VALUE Wrap_FXTableItem(VALUE klass, FXTableItem* ptr, int owned);
extern VALUE Wrap_FXRbTableItem(VALUE klass, FXRbTableItem* ptr, int owned);
extern VALUE Wrap_FXTable(VALUE klass, FXTable* ptr, int owned);
extern VALUE Wrap_FXRbTable(VALUE klass, FXRbTable* ptr, int owned);
extern VALUE Wrap_FXDragCorner(VALUE klass, FXDragCorner* ptr, int owned);
extern VALUE Wrap_FXRbDragCorner(VALUE klass, FXRbDragCorner* ptr, int owned);
extern VALUE Wrap_FXStatusbar(VALUE klass, FXStatusbar* ptr, int owned);
extern VALUE Wrap_FXRbStatusbar(VALUE klass, FXRbStatusbar* ptr, int owned);
extern VALUE Wrap_FXStatusline(VALUE klass, FXStatusline* ptr, int owned);
extern VALUE Wrap_FXRbStatusline(VALUE klass, FXRbStatusline* ptr, int owned);
extern VALUE Wrap_FXMessageBox(VALUE klass, FXMessageBox* ptr, int owned);
extern VALUE Wrap_FXRbMessageBox(VALUE klass, FXRbMessageBox* ptr, int owned);
extern VALUE Wrap_FXDirItem(VALUE klass, FXDirItem* ptr, int owned);
extern VALUE Wrap_FXRbDirItem(VALUE klass, FXRbDirItem* ptr, int owned);
extern VALUE Wrap_FXDirList(VALUE klass, FXDirList* ptr, int owned);
extern VALUE Wrap_FXRbDirList(VALUE klass, FXRbDirList* ptr, int owned);
extern VALUE Wrap_FXSlider(VALUE klass, FXSlider* ptr, int owned);
extern VALUE Wrap_FXRbSlider(VALUE klass, FXRbSlider* ptr, int owned);
extern VALUE Wrap_FXHeaderItem(VALUE klass, FXHeaderItem* ptr, int owned);
extern VALUE Wrap_FXRbHeaderItem(VALUE klass, FXRbHeaderItem* ptr, int owned);
extern VALUE Wrap_FXHeader(VALUE klass, FXHeader* ptr, int owned);
extern VALUE Wrap_FXRbHeader(VALUE klass, FXRbHeader* ptr, int owned);
extern VALUE Wrap_FXShutterItem(VALUE klass, FXShutterItem* ptr, int owned);
extern VALUE Wrap_FXRbShutterItem(VALUE klass, FXRbShutterItem* ptr, int owned);
extern VALUE Wrap_FXShutter(VALUE klass, FXShutter* ptr, int owned);
extern VALUE Wrap_FXRbShutter(VALUE klass, FXRbShutter* ptr, int owned);
extern VALUE Wrap_FXIconItem(VALUE klass, FXIconItem* ptr, int owned);
extern VALUE Wrap_FXRbIconItem(VALUE klass, FXRbIconItem* ptr, int owned);
extern VALUE Wrap_FXIconList(VALUE klass, FXIconList* ptr, int owned);
extern VALUE Wrap_FXRbIconList(VALUE klass, FXRbIconList* ptr, int owned);
extern VALUE Wrap_FXFileItem(VALUE klass, FXFileItem* ptr, int owned);
extern VALUE Wrap_FXRbFileItem(VALUE klass, FXRbFileItem* ptr, int owned);
extern VALUE Wrap_FXFileList(VALUE klass, FXFileList* ptr, int owned);
extern VALUE Wrap_FXRbFileList(VALUE klass, FXRbFileList* ptr, int owned);
extern VALUE Wrap_FXDirBox(VALUE klass, FXDirBox* ptr, int owned);
extern VALUE Wrap_FXRbDirBox(VALUE klass, FXRbDirBox* ptr, int owned);
extern VALUE Wrap_FXDriveBox(VALUE klass, FXDriveBox* ptr, int owned);
extern VALUE Wrap_FXRbDriveBox(VALUE klass, FXRbDriveBox* ptr, int owned);
extern VALUE Wrap_FXDirSelector(VALUE klass, FXDirSelector* ptr, int owned);
extern VALUE Wrap_FXRbDirSelector(VALUE klass, FXRbDirSelector* ptr, int owned);
extern VALUE Wrap_FXFileSelector(VALUE klass, FXFileSelector* ptr, int owned);
extern VALUE Wrap_FXRbFileSelector(VALUE klass, FXRbFileSelector* ptr, int owned);
extern VALUE Wrap_FXColorSelector(VALUE klass, FXColorSelector* ptr, int owned);
extern VALUE Wrap_FXRbColorSelector(VALUE klass, FXRbColorSelector* ptr, int owned);
extern VALUE Wrap_FXFontSelector(VALUE klass, FXFontSelector* ptr, int owned);
extern VALUE Wrap_FXRbFontSelector(VALUE klass, FXRbFontSelector* ptr, int owned);
extern VALUE Wrap_FXText(VALUE klass, FXText* ptr, int owned);
extern VALUE Wrap_FXRbText(VALUE klass, FXRbText* ptr, int owned);
extern VALUE Wrap_FXDataTarget(VALUE klass, FXDataTarget* ptr, int owned);
extern VALUE Wrap_FXRbDataTarget(VALUE klass, FXRbDataTarget* ptr, int owned);
extern VALUE Wrap_FXProgressBar(VALUE klass, FXProgressBar* ptr, int owned);
extern VALUE Wrap_FXRbProgressBar(VALUE klass, FXRbProgressBar* ptr, int owned);
extern VALUE Wrap_FXToolbarGrip(VALUE klass, FXToolbarGrip* ptr, int owned);
extern VALUE Wrap_FXRbToolbarGrip(VALUE klass, FXRbToolbarGrip* ptr, int owned);
extern VALUE Wrap_FXToolbarShell(VALUE klass, FXToolbarShell* ptr, int owned);
extern VALUE Wrap_FXRbToolbarShell(VALUE klass, FXRbToolbarShell* ptr, int owned);
extern VALUE Wrap_FXToolbarTab(VALUE klass, FXToolbarTab* ptr, int owned);
extern VALUE Wrap_FXRbToolbarTab(VALUE klass, FXRbToolbarTab* ptr, int owned);
extern VALUE Wrap_FXDebugTarget(VALUE klass, FXDebugTarget* ptr, int owned);
extern VALUE Wrap_FXRbDebugTarget(VALUE klass, FXRbDebugTarget* ptr, int owned);
extern VALUE Wrap_FXDirDialog(VALUE klass, FXDirDialog* ptr, int owned);
extern VALUE Wrap_FXRbDirDialog(VALUE klass, FXRbDirDialog* ptr, int owned);
extern VALUE Wrap_FXFileDialog(VALUE klass, FXFileDialog* ptr, int owned);
extern VALUE Wrap_FXRbFileDialog(VALUE klass, FXRbFileDialog* ptr, int owned);
extern VALUE Wrap_FXColorDialog(VALUE klass, FXColorDialog* ptr, int owned);
extern VALUE Wrap_FXRbColorDialog(VALUE klass, FXRbColorDialog* ptr, int owned);
extern VALUE Wrap_FXFontDialog(VALUE klass, FXFontDialog* ptr, int owned);
extern VALUE Wrap_FXRbFontDialog(VALUE klass, FXRbFontDialog* ptr, int owned);
extern VALUE Wrap_FXSearchDialog(VALUE klass, FXSearchDialog* ptr, int owned);
extern VALUE Wrap_FXRbSearchDialog(VALUE klass, FXRbSearchDialog* ptr, int owned);
extern VALUE Wrap_FXReplaceDialog(VALUE klass, FXReplaceDialog* ptr, int owned);
extern VALUE Wrap_FXRbReplaceDialog(VALUE klass, FXRbReplaceDialog* ptr, int owned);
extern VALUE Wrap_FXInputDialog(VALUE klass, FXInputDialog* ptr, int owned);
extern VALUE Wrap_FXRbInputDialog(VALUE klass, FXRbInputDialog* ptr, int owned);
extern VALUE Wrap_FXPrintDialog(VALUE klass, FXPrintDialog* ptr, int owned);
extern VALUE Wrap_FXRbPrintDialog(VALUE klass, FXRbPrintDialog* ptr, int owned);
extern VALUE Wrap_FXIcon(VALUE klass, FXIcon* ptr, int owned);
extern VALUE Wrap_FXRbIcon(VALUE klass, FXRbIcon* ptr, int owned);
extern VALUE Wrap_FXBMPIcon(VALUE klass, FXBMPIcon* ptr, int owned);
extern VALUE Wrap_FXRbBMPIcon(VALUE klass, FXRbBMPIcon* ptr, int owned);
extern VALUE Wrap_FXGIFIcon(VALUE klass, FXGIFIcon* ptr, int owned);
extern VALUE Wrap_FXRbGIFIcon(VALUE klass, FXRbGIFIcon* ptr, int owned);
extern VALUE Wrap_FXPNGIcon(VALUE klass, FXPNGIcon* ptr, int owned);
extern VALUE Wrap_FXRbPNGIcon(VALUE klass, FXRbPNGIcon* ptr, int owned);
extern VALUE Wrap_FXXPMIcon(VALUE klass, FXXPMIcon* ptr, int owned);
extern VALUE Wrap_FXRbXPMIcon(VALUE klass, FXRbXPMIcon* ptr, int owned);
extern VALUE Wrap_FXImage(VALUE klass, FXImage* ptr, int owned);
extern VALUE Wrap_FXRbImage(VALUE klass, FXRbImage* ptr, int owned);
extern VALUE Wrap_FXBMPImage(VALUE klass, FXBMPImage* ptr, int owned);
extern VALUE Wrap_FXRbBMPImage(VALUE klass, FXRbBMPImage* ptr, int owned);
extern VALUE Wrap_FXGIFImage(VALUE klass, FXGIFImage* ptr, int owned);
extern VALUE Wrap_FXRbGIFImage(VALUE klass, FXRbGIFImage* ptr, int owned);
extern VALUE Wrap_FXPNGImage(VALUE klass, FXPNGImage* ptr, int owned);
extern VALUE Wrap_FXRbPNGImage(VALUE klass, FXRbPNGImage* ptr, int owned);
extern VALUE Wrap_FXXPMImage(VALUE klass, FXXPMImage* ptr, int owned);
extern VALUE Wrap_FXRbXPMImage(VALUE klass, FXRbXPMImage* ptr, int owned);
extern VALUE Wrap_FXPacker(VALUE klass, FXPacker* ptr, int owned);
extern VALUE Wrap_FXRbPacker(VALUE klass, FXRbPacker* ptr, int owned);
extern VALUE Wrap_FXHorizontalFrame(VALUE klass, FXHorizontalFrame* ptr, int owned);
extern VALUE Wrap_FXRbHorizontalFrame(VALUE klass, FXRbHorizontalFrame* ptr, int owned);
extern VALUE Wrap_FXVerticalFrame(VALUE klass, FXVerticalFrame* ptr, int owned);
extern VALUE Wrap_FXRbVerticalFrame(VALUE klass, FXRbVerticalFrame* ptr, int owned);
extern VALUE Wrap_FXMatrix(VALUE klass, FXMatrix* ptr, int owned);
extern VALUE Wrap_FXRbMatrix(VALUE klass, FXRbMatrix* ptr, int owned);
extern VALUE Wrap_FXGroupBox(VALUE klass, FXGroupBox* ptr, int owned);
extern VALUE Wrap_FXRbGroupBox(VALUE klass, FXRbGroupBox* ptr, int owned);
extern VALUE Wrap_FXSwitcher(VALUE klass, FXSwitcher* ptr, int owned);
extern VALUE Wrap_FXRbSwitcher(VALUE klass, FXRbSwitcher* ptr, int owned);
extern VALUE Wrap_FXSplitter(VALUE klass, FXSplitter* ptr, int owned);
extern VALUE Wrap_FXRbSplitter(VALUE klass, FXRbSplitter* ptr, int owned);
extern VALUE Wrap_FX4Splitter(VALUE klass, FX4Splitter* ptr, int owned);
extern VALUE Wrap_FXRb4Splitter(VALUE klass, FXRb4Splitter* ptr, int owned);
extern VALUE Wrap_FXMDIDeleteButton(VALUE klass, FXMDIDeleteButton* ptr, int owned);
extern VALUE Wrap_FXRbMDIDeleteButton(VALUE klass, FXRbMDIDeleteButton* ptr, int owned);
extern VALUE Wrap_FXMDIRestoreButton(VALUE klass, FXMDIRestoreButton* ptr, int owned);
extern VALUE Wrap_FXRbMDIRestoreButton(VALUE klass, FXRbMDIRestoreButton* ptr, int owned);
extern VALUE Wrap_FXMDIMaximizeButton(VALUE klass, FXMDIMaximizeButton* ptr, int owned);
extern VALUE Wrap_FXRbMDIMaximizeButton(VALUE klass, FXRbMDIMaximizeButton* ptr, int owned);
extern VALUE Wrap_FXMDIMinimizeButton(VALUE klass, FXMDIMinimizeButton* ptr, int owned);
extern VALUE Wrap_FXRbMDIMinimizeButton(VALUE klass, FXRbMDIMinimizeButton* ptr, int owned);
extern VALUE Wrap_FXMDIWindowButton(VALUE klass, FXMDIWindowButton* ptr, int owned);
extern VALUE Wrap_FXRbMDIWindowButton(VALUE klass, FXRbMDIWindowButton* ptr, int owned);
extern VALUE Wrap_FXMDIMenu(VALUE klass, FXMDIMenu* ptr, int owned);
extern VALUE Wrap_FXRbMDIMenu(VALUE klass, FXRbMDIMenu* ptr, int owned);
extern VALUE Wrap_FXMDIClient(VALUE klass, FXMDIClient* ptr, int owned);
extern VALUE Wrap_FXRbMDIClient(VALUE klass, FXRbMDIClient* ptr, int owned);
extern VALUE Wrap_FXMDIChild(VALUE klass, FXMDIChild* ptr, int owned);
extern VALUE Wrap_FXRbMDIChild(VALUE klass, FXRbMDIChild* ptr, int owned);
extern VALUE Wrap_FXDocument(VALUE klass, FXDocument* ptr, int owned);
extern VALUE Wrap_FXRbDocument(VALUE klass, FXRbDocument* ptr, int owned);
extern VALUE Wrap_FXRectangle(VALUE klass, FXRectangle* ptr, int owned);
extern VALUE Wrap_FXPoint(VALUE klass, FXPoint* ptr, int owned);
extern VALUE Wrap_FXSegment(VALUE klass, FXSegment* ptr, int owned);
extern VALUE Wrap_FXArc(VALUE klass, FXArc* ptr, int owned);
extern VALUE Wrap_FXPrinter(VALUE klass, FXPrinter* ptr, int owned);
extern VALUE Wrap_FXEvent(VALUE klass, FXEvent* ptr, int owned);
extern VALUE Wrap_FXVec(VALUE klass, FXVec* ptr, int owned);
extern VALUE Wrap_FXHVec(VALUE klass, FXHVec* ptr, int owned);
extern VALUE Wrap_FXQuat(VALUE klass, FXQuat* ptr, int owned);
extern VALUE Wrap_FXHMat(VALUE klass, FXHMat* ptr, int owned);
extern VALUE Wrap_FXRange(VALUE klass, FXRange* ptr, int owned);
extern VALUE Wrap_FXDVec(VALUE klass, FXDVec* ptr, int owned);
extern VALUE Wrap_FXDHVec(VALUE klass, FXDHVec* ptr, int owned);
extern VALUE Wrap_FXDQuat(VALUE klass, FXDQuat* ptr, int owned);
extern VALUE Wrap_FXDHMat(VALUE klass, FXDHMat* ptr, int owned);
extern VALUE Wrap_FXGLVisual(VALUE klass, FXGLVisual* ptr, int owned);
extern VALUE Wrap_FXRbGLVisual(VALUE klass, FXRbGLVisual* ptr, int owned);
extern VALUE Wrap_FXGLContext(VALUE klass, FXGLContext* ptr, int owned);
extern VALUE Wrap_FXRbGLContext(VALUE klass, FXRbGLContext* ptr, int owned);
extern VALUE Wrap_FXGLCanvas(VALUE klass, FXGLCanvas* ptr, int owned);
extern VALUE Wrap_FXRbGLCanvas(VALUE klass, FXRbGLCanvas* ptr, int owned);
extern VALUE Wrap_FXViewport(VALUE klass, FXViewport* ptr, int owned);
extern VALUE Wrap_FXLight(VALUE klass, FXLight* ptr, int owned);
extern VALUE Wrap_FXMaterial(VALUE klass, FXMaterial* ptr, int owned);
extern VALUE Wrap_FXGLViewer(VALUE klass, FXGLViewer* ptr, int owned);
extern VALUE Wrap_FXRbGLViewer(VALUE klass, FXRbGLViewer* ptr, int owned);
extern VALUE Wrap_FXGLObject(VALUE klass, FXGLObject* ptr, int owned);
extern VALUE Wrap_FXRbGLObject(VALUE klass, FXRbGLObject* ptr, int owned);
extern VALUE Wrap_FXGLGroup(VALUE klass, FXGLGroup* ptr, int owned);
extern VALUE Wrap_FXRbGLGroup(VALUE klass, FXRbGLGroup* ptr, int owned);
extern VALUE Wrap_FXGLShape(VALUE klass, FXGLShape* ptr, int owned);
extern VALUE Wrap_FXRbGLShape(VALUE klass, FXRbGLShape* ptr, int owned);
extern VALUE Wrap_FXPicker(VALUE klass, FXPicker* ptr, int owned);
extern VALUE Wrap_FXRbPicker(VALUE klass, FXRbPicker* ptr, int owned);
extern VALUE Wrap_FXColorBar(VALUE klass, FXColorBar* ptr, int owned);
extern VALUE Wrap_FXRbColorBar(VALUE klass, FXRbColorBar* ptr, int owned);
extern VALUE Wrap_FXColorWheel(VALUE klass, FXColorWheel* ptr, int owned);
extern VALUE Wrap_FXRbColorWheel(VALUE klass, FXRbColorWheel* ptr, int owned);
extern VALUE Wrap_FXICOIcon(VALUE klass, FXICOIcon* ptr, int owned);
extern VALUE Wrap_FXRbICOIcon(VALUE klass, FXRbICOIcon* ptr, int owned);
extern VALUE Wrap_FXICOImage(VALUE klass, FXICOImage* ptr, int owned);
extern VALUE Wrap_FXRbICOImage(VALUE klass, FXRbICOImage* ptr, int owned);
extern VALUE Wrap_FXTGAIcon(VALUE klass, FXTGAIcon* ptr, int owned);
extern VALUE Wrap_FXRbTGAIcon(VALUE klass, FXRbTGAIcon* ptr, int owned);
extern VALUE Wrap_FXTGAImage(VALUE klass, FXTGAImage* ptr, int owned);
extern VALUE Wrap_FXRbTGAImage(VALUE klass, FXRbTGAImage* ptr, int owned);
#ifdef WITH_FXSCINTILLA
extern VALUE Wrap_FXScintilla(VALUE klass, FXScintilla* ptr, int owned);
extern VALUE Wrap_FXRbScintilla(VALUE klass, FXRbScintilla* ptr, int owned);
#endif

extern VALUE cFXRectangle;
extern VALUE cFXPoint;
extern VALUE cFXSegment;
extern VALUE cFXArc;
extern VALUE cFXPrinter;
extern VALUE cFXEvent;
extern VALUE cFXStreamHashEntry;
extern VALUE cFX_Stream;
extern VALUE cFX_FileStream;
extern VALUE cFXMemoryStream;
extern VALUE cFX_Object;
extern VALUE cFXObject;
extern VALUE cFX_Delegator;
extern VALUE cFXDelegator;
extern VALUE cFX_Dict;
extern VALUE cFXDict;
extern VALUE cFX_StringDict;
extern VALUE cFXStringDict;
extern VALUE cFX_Registry;
extern VALUE cFX_AccelTable;
extern VALUE cFXAccelTable;
extern VALUE cFX_RecentFiles;
extern VALUE cFXRecentFiles;
extern VALUE cFXCharset;
extern VALUE cFX_App;
extern VALUE cFXApp;
extern VALUE cFX_Id;
extern VALUE cFXId;
extern VALUE cFX_Visual;
extern VALUE cFXVisual;
extern VALUE cFXFontDesc;
extern VALUE cFX_Font;
extern VALUE cFXFont;
extern VALUE cFX_Cursor;
extern VALUE cFXCursor;
extern VALUE cFX_GIFCursor;
extern VALUE cFXGIFCursor;
extern VALUE cFX_Drawable;
extern VALUE cFXDrawable;
extern VALUE cFX_Bitmap;
extern VALUE cFXBitmap;
extern VALUE cFX_Region;
extern VALUE cFX_DC;
extern VALUE cFXDC;
extern VALUE cFX_DCWindow;
extern VALUE cFXDCWindow;
extern VALUE cFXPSBounds;
extern VALUE cFXDCPrint;
extern VALUE cFXRbDCPrint;
extern VALUE cFXFileAssoc;
extern VALUE cFX_IconDict;
extern VALUE cFXIconDict;
extern VALUE cFX_FileDict;
extern VALUE cFXFileDict;
extern VALUE cFX_Window;
extern VALUE cFXWindow;
extern VALUE cFX_Frame;
extern VALUE cFXFrame;
extern VALUE cFX_HorizontalSeparator;
extern VALUE cFXHorizontalSeparator;
extern VALUE cFX_VerticalSeparator;
extern VALUE cFXVerticalSeparator;
extern VALUE cFX_Label;
extern VALUE cFXLabel;
extern VALUE cFX_Dial;
extern VALUE cFXDial;
extern VALUE cFX_ColorWell;
extern VALUE cFXColorWell;
extern VALUE cFX_TextField;
extern VALUE cFXTextField;
extern VALUE cFX_Button;
extern VALUE cFXButton;
extern VALUE cFX_ToggleButton;
extern VALUE cFXToggleButton;
extern VALUE cFX_CheckButton;
extern VALUE cFXCheckButton;
extern VALUE cFX_RadioButton;
extern VALUE cFXRadioButton;
extern VALUE cFX_ArrowButton;
extern VALUE cFXArrowButton;
extern VALUE cFX_MenuButton;
extern VALUE cFXMenuButton;
extern VALUE cFX_Composite;
extern VALUE cFXComposite;
extern VALUE cFX_Spinner;
extern VALUE cFXSpinner;
extern VALUE cFX_RootWindow;
extern VALUE cFXRootWindow;
extern VALUE cFX_Canvas;
extern VALUE cFXCanvas;
extern VALUE cFX_Shell;
extern VALUE cFXShell;
extern VALUE cFX_Tooltip;
extern VALUE cFXTooltip;
extern VALUE cFX_Popup;
extern VALUE cFXPopup;
extern VALUE cFX_TopWindow;
extern VALUE cFXTopWindow;
extern VALUE cFX_DialogBox;
extern VALUE cFXDialogBox;
extern VALUE cFX_MainWindow;
extern VALUE cFXMainWindow;
extern VALUE cFX_MenuPane;
extern VALUE cFXMenuPane;
extern VALUE cFX_MenuCaption;
extern VALUE cFXMenuCaption;
extern VALUE cFX_MenuSeparator;
extern VALUE cFXMenuSeparator;
extern VALUE cFX_MenuTitle;
extern VALUE cFXMenuTitle;
extern VALUE cFX_MenuCascade;
extern VALUE cFXMenuCascade;
extern VALUE cFX_MenuCommand;
extern VALUE cFXMenuCommand;
extern VALUE cFX_Toolbar;
extern VALUE cFXToolbar;
extern VALUE cFX_Menubar;
extern VALUE cFXMenubar;
extern VALUE cFX_Option;
extern VALUE cFXOption;
extern VALUE cFX_OptionMenu;
extern VALUE cFXOptionMenu;
extern VALUE cFX_TabItem;
extern VALUE cFXTabItem;
extern VALUE cFX_TabBar;
extern VALUE cFXTabBar;
extern VALUE cFX_TabBook;
extern VALUE cFXTabBook;
extern VALUE cFX_Scrollbar;
extern VALUE cFXScrollbar;
extern VALUE cFX_ScrollCorner;
extern VALUE cFXScrollCorner;
extern VALUE cFX_ScrollArea;
extern VALUE cFXScrollArea;
extern VALUE cFX_ScrollWindow;
extern VALUE cFXScrollWindow;
extern VALUE cFX_ListItem;
extern VALUE cFXListItem;
extern VALUE cFX_List;
extern VALUE cFXList;
extern VALUE cFX_ComboBox;
extern VALUE cFXComboBox;
extern VALUE cFX_ListBox;
extern VALUE cFXListBox;
extern VALUE cFX_TreeItem;
extern VALUE cFXTreeItem;
extern VALUE cFX_TreeList;
extern VALUE cFXTreeList;
extern VALUE cFX_TreeListBox;
extern VALUE cFXTreeListBox;
extern VALUE cFX_ImageView;
extern VALUE cFXImageView;
extern VALUE cFXTablePos;
extern VALUE cFXTableRange;
extern VALUE cFX_TableItem;
extern VALUE cFXTableItem;
extern VALUE cFX_Table;
extern VALUE cFXTable;
extern VALUE cFX_DragCorner;
extern VALUE cFXDragCorner;
extern VALUE cFX_Statusbar;
extern VALUE cFXStatusbar;
extern VALUE cFX_Statusline;
extern VALUE cFXStatusline;
extern VALUE cFX_MessageBox;
extern VALUE cFXMessageBox;
extern VALUE cFX_DirItem;
extern VALUE cFXDirItem;
extern VALUE cFX_DirList;
extern VALUE cFXDirList;
extern VALUE cFX_Slider;
extern VALUE cFXSlider;
extern VALUE cFX_HeaderItem;
extern VALUE cFXHeaderItem;
extern VALUE cFX_Header;
extern VALUE cFXHeader;
extern VALUE cFX_ShutterItem;
extern VALUE cFXShutterItem;
extern VALUE cFX_Shutter;
extern VALUE cFXShutter;
extern VALUE cFX_IconItem;
extern VALUE cFXIconItem;
extern VALUE cFX_IconList;
extern VALUE cFXIconList;
extern VALUE cFX_FileItem;
extern VALUE cFXFileItem;
extern VALUE cFX_FileList;
extern VALUE cFXFileList;
extern VALUE cFX_DirBox;
extern VALUE cFXDirBox;
extern VALUE cFX_DriveBox;
extern VALUE cFXDriveBox;
extern VALUE cFX_DirSelector;
extern VALUE cFXDirSelector;
extern VALUE cFX_FileSelector;
extern VALUE cFXFileSelector;
extern VALUE cFX_ColorSelector;
extern VALUE cFXColorSelector;
extern VALUE cFX_FontSelector;
extern VALUE cFXFontSelector;
extern VALUE cFX_Text;
extern VALUE cFXText;
extern VALUE cFXDataTarget;
extern VALUE cFXRbDataTarget;
extern VALUE cFX_ProgressBar;
extern VALUE cFXProgressBar;
extern VALUE cFX_ToolbarGrip;
extern VALUE cFXToolbarGrip;
extern VALUE cFX_ToolbarShell;
extern VALUE cFXToolbarShell;
extern VALUE cFX_ToolbarTab;
extern VALUE cFXToolbarTab;
extern VALUE cFXDebugTarget;
extern VALUE cFXRbDebugTarget;
extern VALUE cFX_DirDialog;
extern VALUE cFXDirDialog;
extern VALUE cFX_FileDialog;
extern VALUE cFXFileDialog;
extern VALUE cFX_ColorDialog;
extern VALUE cFXColorDialog;
extern VALUE cFX_FontDialog;
extern VALUE cFXFontDialog;
extern VALUE cFX_SearchDialog;
extern VALUE cFXSearchDialog;
extern VALUE cFX_ReplaceDialog;
extern VALUE cFXReplaceDialog;
extern VALUE cFX_InputDialog;
extern VALUE cFXInputDialog;
extern VALUE cFX_PrintDialog;
extern VALUE cFXPrintDialog;
extern VALUE cFX_Icon;
extern VALUE cFXIcon;
extern VALUE cFX_BMPIcon;
extern VALUE cFXBMPIcon;
extern VALUE cFX_GIFIcon;
extern VALUE cFXGIFIcon;
extern VALUE cFX_PNGIcon;
extern VALUE cFXPNGIcon;
extern VALUE cFX_XPMIcon;
extern VALUE cFXXPMIcon;
extern VALUE cFX_Image;
extern VALUE cFXImage;
extern VALUE cFX_BMPImage;
extern VALUE cFXBMPImage;
extern VALUE cFX_GIFImage;
extern VALUE cFXGIFImage;
extern VALUE cFX_PNGImage;
extern VALUE cFXPNGImage;
extern VALUE cFX_XPMImage;
extern VALUE cFXXPMImage;
extern VALUE cFX_Packer;
extern VALUE cFXPacker;
extern VALUE cFX_HorizontalFrame;
extern VALUE cFXHorizontalFrame;
extern VALUE cFX_VerticalFrame;
extern VALUE cFXVerticalFrame;
extern VALUE cFX_Matrix;
extern VALUE cFXMatrix;
extern VALUE cFX_GroupBox;
extern VALUE cFXGroupBox;
extern VALUE cFX_Switcher;
extern VALUE cFXSwitcher;
extern VALUE cFX_Splitter;
extern VALUE cFXSplitter;
extern VALUE cFX_4Splitter;
extern VALUE cFX4Splitter;
extern VALUE cFX_MDIDeleteButton;
extern VALUE cFXMDIDeleteButton;
extern VALUE cFX_MDIRestoreButton;
extern VALUE cFXMDIRestoreButton;
extern VALUE cFX_MDIMaximizeButton;
extern VALUE cFXMDIMaximizeButton;
extern VALUE cFX_MDIMinimizeButton;
extern VALUE cFXMDIMinimizeButton;
extern VALUE cFX_MDIWindowButton;
extern VALUE cFXMDIWindowButton;
extern VALUE cFX_MDIMenu;
extern VALUE cFXMDIMenu;
extern VALUE cFX_MDIClient;
extern VALUE cFXMDIClient;
extern VALUE cFX_MDIChild;
extern VALUE cFXMDIChild;
extern VALUE cFX_Document;
extern VALUE cFXDocument;
extern VALUE cFXRectangle;
extern VALUE cFXPoint;
extern VALUE cFXSegment;
extern VALUE cFXArc;
extern VALUE cFXPrinter;
extern VALUE cFXEvent;
extern VALUE cFXVec;
extern VALUE cFXHVec;
extern VALUE cFXQuat;
extern VALUE cFXHMat;
extern VALUE cFXRange;
extern VALUE cFXDVec;
extern VALUE cFXDHVec;
extern VALUE cFXDQuat;
extern VALUE cFXDHMat;
extern VALUE cFX_GLVisual;
extern VALUE cFXGLVisual;
extern VALUE cFX_GLContext;
extern VALUE cFXGLContext;
extern VALUE cFX_GLCanvas;
extern VALUE cFXGLCanvas;
extern VALUE cFXViewport;
extern VALUE cFXLight;
extern VALUE cFXMaterial;
extern VALUE cFX_GLViewer;
extern VALUE cFXGLViewer;
extern VALUE cFX_GLObject;
extern VALUE cFXGLObject;
extern VALUE cFX_GLGroup;
extern VALUE cFXGLGroup;
extern VALUE cFX_GLShape;
extern VALUE cFXGLShape;
extern VALUE cFX_Picker;
extern VALUE cFXPicker;
extern VALUE cFX_ColorBar;
extern VALUE cFXColorBar;
extern VALUE cFX_ColorWheel;
extern VALUE cFXColorWheel;
extern VALUE cFX_ICOIcon;
extern VALUE cFXICOIcon;
extern VALUE cFX_ICOImage;
extern VALUE cFXICOImage;
extern VALUE cFX_TGAIcon;
extern VALUE cFXTGAIcon;
extern VALUE cFX_TGAImage;
extern VALUE cFXTGAImage;
#ifdef WITH_FXSCINTILLA
extern VALUE cFX_Scintilla;
extern VALUE cFXScintilla;
#endif

#endif

