/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// Registers stuff to know about the extension
struct FXFileAssoc {
  FXString   command;           /// Command to execute
  FXString   extension;         /// Full extension name
  FXString   mimetype;          /// Mime type name
  FXIcon    *bigicon;           /// Big normal icon
  FXIcon    *bigiconopen;       /// Big open icon
  FXIcon    *miniicon;          /// Mini normal icon
  FXIcon    *miniiconopen;      /// Mini open icon
  FXDragType dragtype;          /// Registered drag type
  FXuint     flags;             /// Flags
  };


/// Icon dictionary
class FXIconDict : public FXDict {
protected:
  virtual void *createData(const void*);
  virtual void deleteData(void*);
public:

  /// Construct an icon dictionary
  FXIconDict(FXApp* a=NULL);

  /// Get application
  FXApp* getApp() const;

  /// Set icon search path
  void setIconPath(const FXString& p);

  /// Return current icon search path
  FXString getIconPath() const;

  /// Insert unique icon loaded from filename into dictionary
  FXIcon* insert(const FXchar* name);

  /// Remove icon from dictionary
  FXIcon* remove(const FXchar* name);

  /// Find icon by name
  FXIcon* find(const FXchar* name);

  /// Destructor
  virtual ~FXIconDict();

public:
  %pragma(ruby) markfunc = "FXRbIconDict::markfunc";
  };


class FXRbIconDict : public FXIconDict {
public:
  // Construct an icon dictionary
  FXRbIconDict(FXApp* a=NULL);

  // Destructor
  virtual ~FXRbIconDict();

public:
  DECLARE_FXOBJECT_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbIconDict::markfunc";
  };

    
/// File Association dictionary
class FXFileDict : public FXDict {
protected:
  FXIconDict    icons;          // Icon table
public:
#ifdef SWIGRUBY
  %addmethods {
    /// Registry key used to find fallback executable icons
    static VALUE defaultExecBinding(){
      return to_ruby(FXFileDict::defaultExecBinding);
      }

    /// Registry key used to find fallback directory icons
    static VALUE defaultDirBinding(){
      return to_ruby(FXFileDict::defaultDirBinding);
      }

    /// Registry key used to find fallback document icons
    static VALUE defaultFileBinding(){
      return to_ruby(FXFileDict::defaultFileBinding);
      }
  }
#endif
protected:
  virtual void *createData(const void*);
  virtual void deleteData(void*);
public:

  /**
  * Construct a dictionary mapping file-extension to file associations,
  * using the application registry settings as a source for the bindings.
  */
  FXFileDict(FXApp* a=NULL);

  /**
  * Construct a dictionary mapping file-extension to file associations,
  * using the specified settings database as a source for the bindings.
  */
  // FXFileDict(FXApp* a,FXSettings* db);

  /// Get application
  FXApp* getApp() const;

  /// Set icon search path
  void setIconPath(const FXString& path);

  /// Return current icon search path
  FXString getIconPath() const;

  /// Replace file association
  FXFileAssoc* replace(const FXchar* ext,const FXchar* str);

  /// Remove file association
  FXFileAssoc* remove(const FXchar* ext);

  /// Find file association already in dictionary
  FXFileAssoc* find(const FXchar* ext);

  /// Find file association from registry
  FXFileAssoc* associate(const FXchar* key);
  
  /**
  * Determine binding for the given file.
  * The default implementation tries the whole filename first,
  * then tries the extensions.
  * For example, for a file "source.tar.gz":
  *
  *  "source.tar.gz",
  *  "tar.gz",
  *  "gz"
  *
  * are tried in succession.
  *
  * Overload this function if you want to inspect the file's
  * contents in order to determine its type.
  */
  virtual FXFileAssoc* findFileBinding(const FXchar* pathname);
  
  /**
  * Find directory binding from registry.
  * The default implementation tries the whole pathname first,
  * then tries successively smaller parts of the path.
  * For example, a pathname "/usr/people/jeroen":
  *
  *   "/usr/people/jeroen"
  *   "/people/jeroen"
  *   "/jeroen"
  *
  * are tried in succession.
  *
  * Overload this function if you want to determine directory
  * icons in some other way.
  */
  virtual FXFileAssoc* findDirBinding(const FXchar* pathname);

  /**
  * Determine binding for the given executable.
  * The default implementation returns the binding associated with
  * the key "defaultexecbinding".
  */
  virtual FXFileAssoc* findExecBinding(const FXchar* pathname);

  /// Save to stream
  virtual void save(FXStream& store) const;

  /// Load from stream
  virtual void load(FXStream& store);

  /// Destructor
  virtual ~FXFileDict();

public:
  %pragma(ruby) markfunc = "FXRbFileDict::markfunc";
  };


class FXRbFileDict : public FXFileDict {
public:
  /// Construct an file association table
  FXRbFileDict(FXApp* a);

  /**
  * Construct a dictionary mapping file-extension to file associations,
  * using the specified settings database as a source for the bindings.
  */
  // FXRbFileDict(FXApp* a,FXSettings* db);

  // Destructor
  virtual ~FXRbFileDict();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXFILEDICT_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbFileDict::markfunc";
  };

