/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// Icon list styles
enum {
  ICONLIST_EXTENDEDSELECT = 0,                /// Extended selection mode
  ICONLIST_SINGLESELECT   = 0x00100000,       /// At most one selected item
  ICONLIST_BROWSESELECT   = 0x00200000,       /// Always exactly one selected item
  ICONLIST_MULTIPLESELECT = 0x00300000,       /// Multiple selection mode
  ICONLIST_AUTOSIZE       = 0x00400000,       /// Automatically size item spacing
  ICONLIST_DETAILED       = 0,                /// List mode
  ICONLIST_MINI_ICONS     = 0x00800000,       /// Mini Icon mode
  ICONLIST_BIG_ICONS      = 0x01000000,       /// Big Icon mode
  ICONLIST_ROWS           = 0,                /// Row-wise mode
  ICONLIST_COLUMNS        = 0x02000000,       /// Column-wise mode
  ICONLIST_NORMAL         = ICONLIST_EXTENDEDSELECT
  };


class FXIcon;
class FXHeader;
class FXFont;
class FXIconList;


/// Icon item
class FXIconItem : public FXObject {
protected:
  FXString  label;
  FXIcon   *bigIcon;
  FXIcon   *miniIcon;
  void     *data;
  FXuint    state;
protected:
  FXIconItem():bigIcon(NULL),miniIcon(NULL),data(NULL),state(0){}
  virtual void draw(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
  virtual FXint hitItem(const FXIconList* list,FXint rx,FXint ry,FXint rw=1,FXint rh=1) const;
protected:
  virtual void drawBigIcon(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
  virtual void drawMiniIcon(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
  virtual void drawDetails(const FXIconList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h) const;
protected:
  enum {
    SELECTED      = 1,
    FOCUS         = 2,
    DISABLED      = 4,
    DRAGGABLE     = 8,
    BIGICONOWNED  = 16,
    MINIICONOWNED = 32
    };
public:
  // Constructor
  FXIconItem(const FXString& text,FXIcon* bi=NULL,FXIcon* mi=NULL,void* ptr=NULL);

  virtual void setText(const FXString& txt);

  FXString getText() const;

  virtual void setBigIcon(FXIcon* icn);

  FXIcon* getBigIcon() const;

  virtual void setMiniIcon(FXIcon* icn);

  FXIcon* getMiniIcon() const;

  %addmethods {
    void setData(VALUE ptr){
      self->setData((void*) ptr);
      }
    VALUE getData() const {
      return self->getData() ? (VALUE) self->getData() : Qnil;
      }
  }

  virtual void setFocus(FXbool focus);
  FXbool hasFocus() const;

  virtual void setSelected(FXbool selected);

  FXbool isSelected() const;

  virtual void setEnabled(FXbool enabled);

  FXbool isEnabled() const;

  virtual void setDraggable(FXbool draggable);

  FXbool isDraggable() const;

  // virtual void setIconOwned(FXuint owned=(BIGICONOWNED|MINIICONOWNED));
  virtual void setIconOwned(FXuint owned=48);

  FXuint isIconOwned() const;

  virtual FXint getWidth(const FXIconList* list) const;
  virtual FXint getHeight(const FXIconList* list) const;

  virtual void create();
  virtual void detach();
  virtual void destroy();

  virtual void save(FXStream& store) const;
  virtual void load(FXStream& store);

  // virtual ~FXIconItem();

public:
  %pragma(ruby) markfunc = "FXRbIconItem::markfunc";
  };


class FXRbIconItem : public FXIconItem {
public:
  // Constructor
  FXRbIconItem(const FXString& text,FXIcon* bi=NULL,FXIcon* mi=NULL,void* ptr=NULL):label(text),bigIcon(bi),miniIcon(mi),data(ptr),state(0){}

  // Destructor
  %pragma(ruby) freefunc = "FXRbIconItem::freefunc";
  virtual ~FXRbIconItem();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXICONITEM_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbIconItem::markfunc";
  };


/// Icon item collate function
typedef FXint (*FXIconListSortFunc)(const FXIconItem*,const FXIconItem*);


/// Icon List Widget
class FXIconList : public FXScrollArea {
protected:
  FXHeader          *header;            // Header control
  FXIconItem       **items;             // Item list
  FXint              nitems;            // Number of items
  FXint              nrows;             // Number of rows
  FXint              ncols;             // Number of columns
  FXint              anchor;            // Anchor item
  FXint              current;           // Current item
  FXint              extent;            // Extent item
  FXint              cursor;            // Cursor item
  FXFont            *font;              // Font
  FXIconListSortFunc sortfunc;          // Item sort function
  FXColor            textColor;         // Text color
  FXColor            selbackColor;      // Selected back color
  FXColor            seltextColor;      // Selected text color
  FXint              itemWidth;         // Item width
  FXint              itemHeight;        // Item height
  FXint              itemSpace;         // Space for item label
  FXint              anchorx;           // Rectangular selection
  FXint              anchory;
  FXint              currentx;
  FXint              currenty;
  FXint              grabx;             // Grab point x
  FXint              graby;             // Grab point y
  FXString           lookup;            // Lookup string
  FXTimer           *timer;             // Tip hover timer
  FXTimer           *lookuptimer;       // Lookup timer
  FXString           help;              // Help text
  FXbool             state;             // State of item
protected:
  FXIconList();
  void drawLasso(FXint x0,FXint y0,FXint x1,FXint y1);
  virtual void layout();
  void recompute();
  void getrowscols(FXint& nr,FXint& nc,FXint w,FXint h) const;
  virtual FXIconItem *createItem(const FXString& text,FXIcon *big,FXIcon* mini,void* ptr);
  void lassoChanged(FXint ox,FXint oy,FXint ow,FXint oh,FXint nx,FXint ny,FXint nw,FXint nh,FXbool notify);
public:
  long onPaint(FXObject*,FXSelector,void* PTR_EVENT);
  long onEnter(FXObject*,FXSelector,void* PTR_EVENT);
  long onLeave(FXObject*,FXSelector,void* PTR_EVENT);
  long onUngrabbed(FXObject*,FXSelector,void* PTR_EVENT);
  long onKeyPress(FXObject*,FXSelector,void* PTR_EVENT);
  long onKeyRelease(FXObject*,FXSelector,void* PTR_EVENT);
  long onLeftBtnPress(FXObject*,FXSelector,void* PTR_EVENT);
  long onLeftBtnRelease(FXObject*,FXSelector,void* PTR_EVENT);
  long onRightBtnPress(FXObject*,FXSelector,void* PTR_EVENT);
  long onRightBtnRelease(FXObject*,FXSelector,void* PTR_EVENT);
  long onMotion(FXObject*,FXSelector,void* PTR_EVENT);
  long onQueryTip(FXObject*,FXSelector,void* PTR_IGNORE);
  long onQueryHelp(FXObject*,FXSelector,void* PTR_IGNORE);
  long onTipTimer(FXObject*,FXSelector,void* PTR_IGNORE);
  long onCmdSelectAll(FXObject*,FXSelector,void* PTR_IGNORE);
  long onCmdDeselectAll(FXObject*,FXSelector,void* PTR_IGNORE);
  long onCmdSelectInverse(FXObject*,FXSelector,void* PTR_IGNORE);
  long onCmdArrangeByRows(FXObject*,FXSelector,void* PTR_IGNORE);
  long onUpdArrangeByRows(FXObject*,FXSelector,void* PTR_IGNORE); // FIXME
  long onCmdArrangeByColumns(FXObject*,FXSelector,void* PTR_IGNORE);
  long onUpdArrangeByColumns(FXObject*,FXSelector,void* PTR_IGNORE); // FIXME
  long onCmdShowDetails(FXObject*,FXSelector,void* PTR_IGNORE);
  long onUpdShowDetails(FXObject*,FXSelector,void* PTR_IGNORE); // FIXME
  long onCmdShowBigIcons(FXObject*,FXSelector,void* PTR_IGNORE);
  long onUpdShowBigIcons(FXObject*,FXSelector,void* PTR_IGNORE); // FIXME
  long onCmdShowMiniIcons(FXObject*,FXSelector,void* PTR_IGNORE);
  long onUpdShowMiniIcons(FXObject*,FXSelector,void* PTR_IGNORE); // FIXME
  long onHeaderChanged(FXObject*,FXSelector,void* PTR_IGNORE);
  long onFocusIn(FXObject*,FXSelector,void* PTR_EVENT);
  long onFocusOut(FXObject*,FXSelector,void* PTR_EVENT);
  long onClicked(FXObject*,FXSelector,void* PTR_INT);
  long onDoubleClicked(FXObject*,FXSelector,void* PTR_INT);
  long onTripleClicked(FXObject*,FXSelector,void* PTR_INT);
  long onCommand(FXObject*,FXSelector,void* PTR_INT);
  long onAutoScroll(FXObject*,FXSelector,void* PTR_EVENT);
  long onLookupTimer(FXObject*,FXSelector,void* PTR_IGNORE);
  long onCmdSetValue(FXObject*,FXSelector,void* PTR_INT);
  long onCmdGetIntValue(FXObject*,FXSelector,void* PTR_IGNORE); // FIXME
  long onCmdSetIntValue(FXObject*,FXSelector,void* PTR_PINT);
public:
  static FXint ascending(const FXIconItem* a,const FXIconItem* b);
  static FXint descending(const FXIconItem* a,const FXIconItem* b);
public:
  enum {
    ID_SHOW_DETAILS=FXScrollArea::ID_LAST,
    ID_SHOW_MINI_ICONS,
    ID_SHOW_BIG_ICONS,
    ID_ARRANGE_BY_ROWS,
    ID_ARRANGE_BY_COLUMNS,
    ID_HEADER_CHANGE,
    ID_TIPTIMER,
    ID_LOOKUPTIMER,
    ID_SELECT_ALL,
    ID_DESELECT_ALL,
    ID_SELECT_INVERSE,
    ID_LAST
    };
public:
  /// Construct icon list
  FXIconList(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=ICONLIST_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  /// Return number of items
  FXint getNumItems() const;
  
  /// Return number of rows
  FXint getNumRows() const;
  
  /// Return number of columns
  FXint getNumCols() const;

  /// Return header control
  FXHeader* getHeader() const;

  /// Append header with given text and optional icon
  void appendHeader(const FXString& text,FXIcon *icon=NULL,FXint size=1);
  
  /// Remove header at index
  void removeHeader(FXint index);
  
  /// Change text of header at index
  void setHeaderText(FXint index,const FXString& text);
  
  /// Return text of header at index
  FXString getHeaderText(FXint index) const;
  
  /// Change icon of header at index
  void setHeaderIcon(FXint index,FXIcon *icon);
  
  /// Return icon of header at index
  FXIcon* getHeaderIcon(FXint index) const;
  
  /// Change size of header at index
  void setHeaderSize(FXint index,FXint size);
  
  /// Return width of header at index
  FXint getHeaderSize(FXint index) const;
  
  /// Return number of headers
  FXint getNumHeaders() const;
  
  /// Return the item at the given index
  FXIconItem *retrieveItem(FXint index) const;
  
  /// Replace the item with a [possibly subclassed] item
  %addmethods {
    FXint replaceItem1(FXint index,FXIconItem* item,FXbool notify=FALSE){
      if(item->isMemberOf(FXMETACLASS(FXRbIconItem)))
        ((FXRbIconItem*)item)->owned=TRUE;
      return self->replaceItem(index,item,notify);
      }
  }
  
  /// Replace items text, icons, and user-data pointer
  %name(replaceItem2) FXint replaceItem(FXint index,const FXString& text,FXIcon *big=NULL,FXIcon* mini=NULL,void* ITEMDATA=NULL,FXbool notify=FALSE);

  // Placeholder for overloaded replaceItem()
  FXint replaceItem(FXint index,FXIconItem* item);
  
  %addmethods {
    /// Insert a new [possibly subclassed] item at the give index
    FXint insertItem1(FXint index,FXIconItem* item,FXbool notify=FALSE){
      if(item->isMemberOf(FXMETACLASS(FXRbIconItem)))
        ((FXRbIconItem*)item)->owned=TRUE;
      return self->insertItem(index,item,notify);
      }
  }
  
  /// Insert item at index with given text, icons, and user-data pointer
  %name(insertItem2) FXint insertItem(FXint index,const FXString& text,FXIcon *big=NULL,FXIcon* mini=NULL,void* ITEMDATA=NULL,FXbool notify=FALSE);
  
  // Placeholder for overloaded insertItem()
  FXint insertItem(FXint index,FXIconItem* item,FXbool notify=FALSE);

  %addmethods {
    /// Append a [possibly subclassed] item to the end of the list
    FXint appendItem1(FXIconItem* item,FXbool notify=FALSE){
      if(item->isMemberOf(FXMETACLASS(FXRbIconItem)))
        ((FXRbIconItem*)item)->owned=TRUE;
      return self->appendItem(item,notify);
      }
  }
  
  /// Append new item with given text and optional icons, and user-data pointer
  %name(appendItem2) FXint appendItem(const FXString& text,FXIcon *big=NULL,FXIcon* mini=NULL,void* ITEMDATA=NULL,FXbool notify=FALSE);

  // Placeholder for overloaded appendItem()
  FXint appendItem(FXIconItem* item,FXbool notify=FALSE);
  
  %addmethods {
    /// Append a [possibly subclassed] item to the end of the list
    FXint prependItem1(FXIconItem* item,FXbool notify=FALSE){
      if(item->isMemberOf(FXMETACLASS(FXRbIconItem)))
        ((FXRbIconItem*)item)->owned=TRUE;
      return self->prependItem(item,notify);
      }
  }
  
  /// Append new item with given text and optional icons, and user-data pointer
  %name(prependItem2) FXint prependItem(const FXString& text,FXIcon *big=NULL,FXIcon* mini=NULL,void* ITEMDATA=NULL,FXbool notify=FALSE);

  // Placeholder for overloaded prependItem()
  FXint prependItem(FXIconItem* item,FXbool notify=FALSE);

  /// Remove item from list
  void removeItem(FXint index,FXbool notify=FALSE);
  
  /// Remove all items from list
  void clearItems(FXbool notify=FALSE);
  
  /// Return item width
  FXint getItemWidth() const;

  /// Return item height
  FXint getItemHeight() const;

  /// Return index of item at x,y, or -1 if none
  FXint getItemAt(FXint x,FXint y) const;
  
  /** 
  * Search items for item by name, starting from start item; the
  * flags argument controls the search direction, and case sensitivity.
  */
  FXint findItem(const FXString& text,FXint start=-1,FXuint flags=SEARCH_FORWARD|SEARCH_WRAP) const;
  
  /// Scroll to make item at index visible
  void makeItemVisible(FXint index);
  
  /// Change item text
  void setItemText(FXint index,const FXString& text);
  
  /// Return item text
  FXString getItemText(FXint index) const;
  
  /// Change item big icon
  void setItemBigIcon(FXint index,FXIcon* icon);
  
  /// Return big icon of item at index
  FXIcon* getItemBigIcon(FXint index) const;
  
  /// Change item mini icon
  void setItemMiniIcon(FXint index,FXIcon* icon);
  
  /// Return mini icon of item at index
  FXIcon* getItemMiniIcon(FXint index) const;
  
  %addmethods {
    /// Change item user-data pointer
    void setItemData(FXint index, VALUE ptr){
      self->setItemData(index, (void*) ptr);
      }
  
    /// Return item user-data pointer
    VALUE getItemData(FXint index) const {
      return self->getItemData(index) ? (VALUE) self->getItemData(index) : Qnil;
      }
  }
  
  /// Return TRUE if item at index is selected
  FXbool isItemSelected(FXint index) const;
  
  /// Return TRUE if item at index is current
  FXbool isItemCurrent(FXint index) const;
  
  /// Return TRUE if item at index is visible
  FXbool isItemVisible(FXint index) const;
  
  /// Return TRUE if item at index is enabled
  FXbool isItemEnabled(FXint index) const;
  
  /// Return item hit code: 0 outside, 1 icon, 2 text
  FXint hitItem(FXint index,FXint x,FXint y,FXint ww=1,FXint hh=1) const;
  
  /// Repaint item at index
  void updateItem(FXint index);
  
  /// Select items in rectangle
  FXbool selectInRectangle(FXint x,FXint y,FXint w,FXint h,FXbool notify=FALSE);
  
  /// Enable item at index
  FXbool enableItem(FXint index);
  
  /// Disable item at index
  FXbool disableItem(FXint index);
  
  /// Select item at index
  FXbool selectItem(FXint index,FXbool notify=FALSE);
  
  /// Deselect item at index
  FXbool deselectItem(FXint index,FXbool notify=FALSE);
  
  /// Toggle item at index
  FXbool toggleItem(FXint index,FXbool notify=FALSE);
  
  /// Change current item index
  void setCurrentItem(FXint index,FXbool notify=FALSE);
  
  /// Return current item index, or -1 if none
  FXint getCurrentItem() const;

  /// Change anchor item index
  void setAnchorItem(FXint index);
  
  /// Return anchor item index, or -1 if none
  FXint getAnchorItem() const { return anchor; }

  /// Return index of item under cursor, or -1 if none
  FXint getCursorItem() const;

  /// Extend selection from anchor index to index 
  FXbool extendSelection(FXint index,FXbool notify=FALSE);
  
  /// Deselect all items
  FXbool killSelection(FXbool notify=FALSE);
  
  /// Sort items
  void sortItems();
  
  /// Change text font
  void setFont(FXFont* fnt);
  
  /// Return text font
  FXFont* getFont() const;

  /// Return normal text color
  FXColor getTextColor() const;

  /// Change normal text color
  void setTextColor(FXColor clr);
  
  /// Return selected text background
  FXColor getSelBackColor() const;

  /// Change selected text background
  void setSelBackColor(FXColor clr);
  
  /// Return selected text color
  FXColor getSelTextColor() const;

  /// Change selected text color
  void setSelTextColor(FXColor clr);
  
  /// Change maximum item space for each item
  void setItemSpace(FXint s);
  
  /// Return maximum item space
  FXint getItemSpace() const;

  %addmethods {
    /// Return sort function
    VALUE getSortFunc() const {
      return Qnil;
      }
  }

  %addmethods {
    /// Change sort function
    void setSortFunc(VALUE proc){
      }
  }

  /// Get the current icon list style
  FXuint getListStyle() const;

  /// Set the current icon list style.
  void setListStyle(FXuint style);

  /// Set the status line help text for this widget
  void setHelpText(const FXString& text);

  /// Get the status line help text for this widget
  FXString getHelpText() const;

  /// Destructor
  virtual ~FXIconList();

public:
  %pragma(ruby) markfunc = "FXRbIconList::markfunc";
  };


class FXRbIconList : public FXIconList {
public:
  // Constructor
  FXRbIconList(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=ICONLIST_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  // Destructor
  virtual ~FXRbIconList();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXID_VIRTUALS
  DECLARE_FXDRAWABLE_VIRTUALS
  DECLARE_FXWINDOW_VIRTUALS
  DECLARE_FXSCROLLAREA_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbIconList::markfunc";
  };

