/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

// Spinner Options
enum {
  SPIN_NORMAL  =  0,                // Normal, non-cyclic
  SPIN_CYCLIC  =  0x00020000,       // Cyclic spinner
  SPIN_NOTEXT  =  0x00040000,       // No text visible
  SPIN_NOMAX   =  0x00080000,       // Spin all the way up to infinity
  SPIN_NOMIN   =  0x00100000        // Spin all the way down to -infinity
  };


class FXTextField;
class FXArrowButton;


/// Spinner control
class FXSpinner : public FXPacker {
protected:
  FXTextField   *textField;         // Text field
  FXArrowButton *upButton;          // The up button
  FXArrowButton *downButton;        // The down button
  FXint          range[2];          // Reported data range
  FXint          incr;              // Increment
  FXint          pos;               // Current position
protected:
  FXSpinner();
  virtual void layout();
public:
  long onUpdIncrement(FXObject*,FXSelector,void* PTR_IGNORE);
  long onCmdIncrement(FXObject*,FXSelector,void* PTR_EVENT);
  long onUpdDecrement(FXObject*,FXSelector,void* PTR_IGNORE);
  long onCmdDecrement(FXObject*,FXSelector,void* PTR_EVENT);
  long onCmdEntry(FXObject*,FXSelector,void* PTR_IGNORE);
  long onChgEntry(FXObject*,FXSelector,void* PTR_IGNORE);
  long onKeyPress(FXObject*,FXSelector,void* PTR_EVENT);
  long onKeyRelease(FXObject*,FXSelector,void* PTR_EVENT);
  long onCmdSetValue(FXObject*,FXSelector,void* PTR_INT);
  long onCmdSetIntValue(FXObject*,FXSelector,void* PTR_PINT);
  long onCmdGetIntValue(FXObject*,FXSelector,void* PTR_IGNORE); // FIXME
  long onCmdSetIntRange(FXObject*,FXSelector,void* PTR_INTRANGE_IN);
  long onCmdGetIntRange(FXObject*,FXSelector,void* PTR_INTRANGE_OUT);
public:
  enum{
    ID_INCREMENT=FXPacker::ID_LAST,
    ID_DECREMENT,
    ID_ENTRY,
    ID_LAST
    };
public:

  /// Construct a spinner
  FXSpinner(FXComposite *p,FXint cols,FXObject *tgt=NULL,FXSelector sel=0,FXuint opts=SPIN_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);

  /// Increment spinner
  void increment();
  
  /// Decrement spinner
  void decrement();
  
  /// Return TRUE if in cyclic mode
  FXbool isCyclic() const;
  
  /// Set to cyclic mode, i.e. wrap around at maximum/minimum
  void setCyclic(FXbool cyclic);
  
  /// Return TRUE if text is visible
  FXbool isTextVisible() const;
  
  /// Set text visible flag
  void setTextVisible(FXbool shown);
  
  /// Change current value
  virtual void setValue(FXint value);
  
  /// Return current value
  FXint getValue() const;

  /// Change the spinner's range
  void setRange(FXint lo,FXint hi);
  
#ifdef SWIGRUBY
  %addmethods {
    /// Get the spinner's current range
    VALUE getRange() const {
        FXint lo, hi;
        self->getRange(lo, hi);
        VALUE result = rb_ary_new();
        rb_ary_push(result, INT2NUM(lo));
        rb_ary_push(result, INT2NUM(hi));
        return result;
    }
  }
#endif
#ifdef SWIGPYTHON
  %addmethods {
    PyObject* getRange() const {
      FXint lo,hi;
      self->getRange(lo,hi);
      FXbool doSave = FXPyRestoreThread();
      PyObject* tuple=PyTuple_New(2);
      if (tuple) {
	PyObject *obj1 = PyInt_FromLong(lo);
	if (!obj1 || PyTuple_SetItem(tuple, 0, obj1) < 0) {
	  Py_DECREF(tuple);
          FXPySaveThread(doSave);
	  return NULL;
	}
	PyObject *obj2 = PyInt_FromLong(hi);
	if (!obj2 || PyTuple_SetItem(tuple, 1, obj2) < 0) {
	  Py_DECREF(tuple);
          FXPySaveThread(doSave);
	  return NULL;
	}
      }
      FXPySaveThread(doSave);
      return tuple;
    }
  }
#endif

  /// Change spinner increment
  void setIncrement(FXint increment);
  
  /// Return spinner increment
  FXint getIncrement() const;

  /// Set the text font
  void setFont(FXFont *fnt);

  /// Get the text font
  FXFont *getFont() const;

  /// Set the status line help text for this spinner
  void setHelpText(const FXString& text);

  /// Get the status line help text for this spinner
  FXString getHelpText() const;

  /// Set the tool tip message for this spinner
  void setTipText(const FXString& text);

  /// Get the tool tip message for this spinner
  FXString getTipText() const;

  /// Change spinner style
  void setSpinnerStyle(FXuint style);
  
  /// Return current spinner style
  FXuint getSpinnerStyle() const;

  /// Allow editing of the text field
  void setEditable(FXbool edit=TRUE);
  
  /// Return TRUE if text field is editable
  FXbool isEditable() const;
  
  /// Destructor
  virtual ~FXSpinner();

public:
  %pragma(ruby) markfunc = "FXRbSpinner::markfunc";
  };


class FXRbSpinner : public FXSpinner {
public:
  // Construct a spinner
  FXRbSpinner(FXComposite *p,FXint cols,FXObject *tgt=NULL,FXSelector sel=0,FXuint opts=SPIN_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);
  %pragma(python) addtomethod = "__init__:FXPyRegister(self)"

  // Destructor
  virtual ~FXRbSpinner();

public:
  DECLARE_FXOBJECT_VIRTUALS
  DECLARE_FXID_VIRTUALS
  DECLARE_FXDRAWABLE_VIRTUALS
  DECLARE_FXWINDOW_VIRTUALS
  DECLARE_FXSPINNER_VIRTUALS

public:
  %pragma(ruby) markfunc = "FXRbSpinner::markfunc";
  };

