/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/// Stream data flow direction
enum FXStreamDirection { 
  FXStreamDead=0,               // Unopened stream
  FXStreamSave=1,               // Saving stuff to stream
  FXStreamLoad=2                // Loading stuff from stream
  };

    
/// Stream status codes
enum FXStreamStatus {
  FXStreamOK=0,                 // OK
  FXStreamEnd=1,                // Try read past end of stream
  FXStreamFull=2,               // Filled up stream buffer or disk full
  FXStreamNoWrite=3,            // Unable to open for write
  FXStreamNoRead=4,             // Unable to open for read
  FXStreamFormat=5,             // Stream format error
  FXStreamUnknown=6,            // Trying to read unknown class
  FXStreamAlloc=7,              // Alloc failed
  FXStreamFailure=8             // General failure
  };


// Hash table entry
struct FXStreamHashEntry { 
  FXuint    ref;                // Object reference number
  FXObject* obj;                // Pointer to object
  };


/*********************  Persistent Store Definition  *************************/

/// Persistent store definition
class FXStream {
protected:
  FXStreamDirection  dir;       // Direction of current transfer
  FXStreamStatus     code;      // Status code
  unsigned long      pos;       // Position
  
protected:

  /// Save bunch of items
  virtual void saveItems(const void *buf,FXuint n);

  /// Load bunch of items
  virtual void loadItems(void *buf,FXuint n);

public:

  /// Constructor
  FXStream(const FXObject* cont=NULL);

  /// Open archive return TRUE if OK
  FXbool open(FXStreamDirection save_or_load);

  /// Close; return TRUE if OK
  FXbool close();

  /// Get status code
  FXStreamStatus status() const;

  /// Set status code
  void setError(FXStreamStatus err);

  /// Obtain direction
  FXStreamDirection direction() const;

  /// Get parent object
  const FXObject* container() const;

  /// Get position
  %name(getPosition) unsigned long position() const;

  /// Move to position
  %name(setPosition) virtual FXbool position(unsigned long p);

  /// Change swap bytes flag
  %name(setSwapBytes) void swapBytes(FXbool s);

  /// Get swap bytes flag
  %name(getSwapBytes) FXbool swapBytes() const;

  /// Return implementation's endianness
  static FXbool isLittleEndian();
  
  /// Destructor
  virtual ~FXStream();

public:
  %pragma(ruby) markfunc = "FXRbStream::markfunc";
  };



/***************************  File Store Definition  ***************************/

/// File Store Definition
class FXFileStream : public FXStream {
protected:
  virtual void saveItems(const void *buf,FXuint n);
  virtual void loadItems(void *buf,FXuint n);

public:

  /// Create file store
  FXFileStream(const FXObject* cont=NULL);

  // Open the stream (instance method)
  FXbool open(const FXString& filename,FXStreamDirection save_or_load);

  /// Move to position
  %name(setPosition) virtual FXbool position(unsigned long p);
  
  /// Destructor
  virtual ~FXFileStream();

public:
  %pragma(ruby) markfunc = "FXRbFileStream::markfunc";
  };

  
/**************************  Memory Store Definition  **************************/

/// Memory Store Definition
class FXMemoryStream : public FXStream {
protected:
  virtual void saveItems(const void *buf,FXuint n);
  virtual void loadItems(void *buf,FXuint n);

public:

  /// Create memory store
  FXMemoryStream(const FXObject* cont=NULL);

  /// Open file store
  FXbool open(FXuchar* data,FXStreamDirection save_or_load);
  
  /// Open memory store
  // FXbool open(FXuchar* data,FXuint sp,FXStreamDirection save_or_load); OVERLOAD

  /// Get available space
  FXuint getSpace() const { return space; }

  /// Set available space
  void setSpace(FXuint sp);
  
#ifdef SWIGRUBY
  %addmethods {
    // Take buffer away from stream
    VALUE takeBuffer() {
      FXuchar* buffer;
      FXuint sp;
      self->takeBuffer(buffer, sp);
      VALUE result = rb_str_new((const char *) buffer, sp);
      FXFREE(&buffer);
      return result;
    }
  }
#endif
  
  /// Give buffer to stream
  void giveBuffer(FXuchar *buffer,FXuint sp);
  
  /// Move to position
  %name(setPosition) virtual FXbool position(unsigned long p);

  /// Destructor
  virtual ~FXMemoryStream();

public:
  %pragma(ruby) markfunc = "FXRbMemoryStream::markfunc";
  };
