/***********************************************************************
 * FXRuby -- the Ruby language bindings for the FOX GUI toolkit.
 * Copyright (c) 2001 by J. Lyle Johnson. All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * For further information please contact the author by e-mail
 * at "lyle@users.sourceforge.net".
 ***********************************************************************/

/***********************************************************************
 * $Id: typemaps.i,v 1.27 2002/01/17 23:52:57 ljohnson Exp $
 ***********************************************************************/

/* Convert Ruby VALUE to VALUE */
%typemap(ruby, out) VALUE "$target = $source;";

/* Convert Ruby string to FXString */
%typemap(ruby, in) const FXString& {
  if ($source != Qnil) {
    Check_Type($source, T_STRING);
    $target = new FXString(STR2CSTR($source));
  } else {
    $target = new FXString;
  }
}

%typemap(ruby, freearg) const FXString& "delete $source;";

/* Convert Ruby true and false to FXbool inputs */
%typemap(ruby, in) FXbool {
  if ($source == Qtrue || $source == Qfalse)
    $target = ($source == Qtrue) ? TRUE : FALSE;
  else
    rb_raise(rb_eTypeError, "wrong argument type (expected true or false)");
}

/* Convert FXString return values to Ruby strings */
%typemap(ruby, out) FXString "$target = rb_str_new2($source->text());";
%typemap(ruby, ret) FXString "delete $source;";

/* Convert const FXString& return values to Ruby strings */
%typemap(ruby, out) const FXString& "$target = rb_str_new2($source->text());";

/* Convert FXbool return types */
%typemap(ruby, out) FXbool "$target = $source ? Qtrue : Qfalse;";

/* Convert void pointers */
%typemap(ruby, in) void* "$target = FXRbGetVoidPointer($source);";

/**
 * Used by constructors for icons and images that require an array
 * of bytes as input; uses a Ruby string instead. Also accepts "nil"
 * in place of a NULL pointer.
 */
%typemap(ruby, in) const void* pix {
  if ($source != Qnil) {
    Check_Type($source, T_STRING);
    $target = (void*) RSTRING($source)->ptr;
  } else {
    $target = NULL;
  }
}

/* Same as above, but when the pixels array is typed as unsigned chars */
%typemap(ruby, in) const FXuchar* PIXELS {
  if ($source != Qnil) {
    Check_Type($source, T_STRING);
    $target = (FXuchar *) RSTRING($source)->ptr;
  } else {
    $target = NULL;
  }
}

/**
 * Used by constructors for FXXPMIcon and FXXPMImage, which require an array
 * of strings as input; uses a Ruby list of strings instead. Also accepts "nil"
 * in place of a NULL pointer.
 */
%typemap(ruby, in) const FXchar** pix {
  $target = NULL;
  if ($source != Qnil) {
    Check_Type($source, T_ARRAY);
    if (FXMALLOC(&$target, FXchar *, RARRAY($source)->len)) {
      for (long i = 0; i < RARRAY($source)->len; i++) {
        $target[i] = (FXchar *) STR2CSTR(rb_ary_entry($source, i));
      }
    }
  }
}

/**
 * Free-up the temporary array allocated above.
 */
%typemap(ruby, freearg) const FXchar** pix "FXFREE(&$source);";

/**
 * Convert a Ruby I/O object of some kind into a platform-specific
 * FXInputHandle for Win32 or Unix. The input can be any object that
 * responds to the "fileno" method.
 */
%typemap(ruby, in) FXInputHandle {
  if(rb_respond_to($source, rb_intern("fileno"))==Qtrue){
    $target=FXRbGetInputHandle($source);
    }
  else{
    rb_raise(rb_eTypeError,"addInput and removeInput require I/O objects");
    }
}

/* Convert a Ruby object into a void pointer (for list item data) */
%typemap(ruby, in) void* ITEMDATA "$target = ($source == Qnil) ? NULL : (void *) $source;";

/* Convert a Ruby FXEvent instance into a pointer to a C++ FXEvent */
%typemap(ruby, in) void* PTR_EVENT(FXEvent *pEvent) {
  Get_FXEvent($source, pEvent);
  $target = (void *) pEvent;
}

/* Convert a Ruby FXObject instance into a pointer to a C++ FXObject */
%typemap(ruby, in) void* PTR_OBJECT(FXObject *pObject) {
  Get_FXObject($source, pObject);
  $target = (void *) pObject;
}

/* Convert a Ruby FXDirItem instance into a pointer to a C++ FXDirItem */
%typemap(ruby, in) void* PTR_DIRITEM(FXDirItem *pDirItem) {
  Get_FXDirItem($source, pDirItem);
  $target = (void *) pDirItem;
}

/* Convert a Ruby FXTreeItem instance into a pointer to a C++ FXTreeItem */
%typemap(ruby, in) void* PTR_TREEITEM(FXTreeItem *pTreeItem) {
  Get_FXTreeItem($source, pTreeItem);
  $target = (void *) pTreeItem;
}

/* Convert a Ruby number into an FXint */
%typemap(ruby, in) void* PTR_INT(FXint value) {
  value = (FXint) NUM2INT($source);
  $target = (void *) value;
}

/* Convert a Ruby number into a pointer to an FXint */
%typemap(ruby, in) void* PTR_PINT(FXint value) {
  value = (FXint) NUM2INT($source);
  $target = (void *) &value;
}

/* Convert a Ruby array (size 2) into an FXint array */
%typemap(ruby, in) void* PTR_INTRANGE_IN(FXint values[2]) {
  Check_Type($source, T_ARRAY);
  values[0] = (FXint) NUM2INT(rb_ary_entry($source, 0));
  values[1] = (FXint) NUM2INT(rb_ary_entry($source, 1));
  $target = (void *) values;
}

/* Convert an FXint array (size 2) into a Ruby array */
%typemap(ruby, freeargs) void* PTR_INTRANGE_OUT(FXint* values) {
  Check_Type($source, T_ARRAY);
  values = (FXint *) $target;
  rb_ary_store($source, 0, to_ruby(values[0]));
  rb_ary_store($source, 1, to_ruby(values[1]));
}

/* Convert a Ruby number into an FXuint */
%typemap(ruby, in) void* PTR_UINT(FXuint value) {
  value = (FXuint) NUM2UINT($source);
  $target = (void *) value;
}

/* Convert a Ruby number into an FXuchar */
%typemap(ruby, in) void* PTR_UCHAR(FXuchar value) {
  value = (FXuchar) NUM2UINT($source);
  $target = (void *) value;
}

/* Convert a Ruby number into an FXColor */
%typemap(ruby, in) void* PTR_COLOR(FXColor value) {
  value = (FXColor) NUM2UINT($source);
  $target = (void *) value;
}

/* Convert a Ruby instance into a void pointer to a C++ NULL */
%typemap(ruby, in) void* PTR_NULL {
  $target = NULL;
}

/* Message data is ignored (doesn't matter what we set $target to) */
%typemap(ruby, in) void* PTR_IGNORE {
  $target = NULL;
}

/* Convert a Ruby string to a C string */
%typemap(ruby, in) void* PTR_CSTRING {
  $target = (void *) STR2CSTR($source);
}

/* Convert a Ruby string to a pointer to an FXString */
%typemap(ruby, in) void* PTR_STRING(FXString value) {
  value = FXString(STR2CSTR($source));
  $target = (void *) &value;
}

/* Convert a Ruby number into a pointer to an FXdouble */
%typemap(ruby, in) void* PTR_PDOUBLE(FXdouble value) {
  value = (FXdouble) NUM2DBL($source);
  $target = (void *) &value;
}

/* Convert a Ruby array (of size 2) into an FXdouble array */
%typemap(ruby, in) void* PTR_DBLRANGE_IN(FXdouble values[2]) {
  Check_Type($source, T_ARRAY);
  values[0] = (FXdouble) NUM2DBL(rb_ary_entry($source, 0));
  values[1] = (FXdouble) NUM2DBL(rb_ary_entry($source, 1));
  $target = (void *) values;
}

/* Convert an FXdouble array to a Ruby array */
%typemap(ruby, freeargs) void* PTR_DBLRANGE_OUT(FXdouble* values) {
  Check_Type($source, T_ARRAY);
  values = (FXdouble*) $target;
  rb_ary_store($source, 0, to_ruby(values[0]));
  rb_ary_store($source, 1, to_ruby(values[1]));
}

/* Convert a Ruby instance into either a NULL pointer or a non-NULL */
%typemap(ruby, in) void* PTR_BOOL {
  $target = NIL_P($source) ? (void*)0 : (void*)1;
}

/* Convert a Ruby array (of size 3) into an FXfloat array */
%typemap(ruby, in) void* PTR_FLTARRAY(FXfloat values[3]) {
  Check_Type($source, T_ARRAY);
  values[0] = (FXfloat) NUM2DBL(rb_ary_entry($source, 0));
  values[1] = (FXfloat) NUM2DBL(rb_ary_entry($source, 1));
  values[2] = (FXfloat) NUM2DBL(rb_ary_entry($source, 2));
  $target = (void *) values;
}

/* Convert a Ruby FXPoint instance into a pointer to a C++ FXPoint */
%typemap(ruby, in) void* PTR_POINT(FXPoint *point) {
  Get_FXPoint($source, point);
  $target = (void *) point;
}

%typemap(ruby, in) void* PTR_MAGIC {
  $target = FXRbGetExpectedData(self, arg2, $source);
}

/* Output typemap for FXAccelTable instances */
%typemap(ruby, out) FXAccelTable* "$target = FXRbWrapClassInstance(cFX_AccelTable, $source);";

/* Output typemap for FXApp instances */
%typemap(ruby, out) FXApp* "$target = FXRbWrapClassInstance(cFX_App, $source);";

/* Output typemap for FXBitmap instances */
%typemap(ruby, out) FXBitmap* "$target = FXRbWrapClassInstance(cFX_Bitmap, $source);";

/* Output typemap for FXButton instances */
%typemap(ruby, out) FXButton* "$target = FXRbWrapClassInstance(cFX_Button, $source);";

/* Output typemap for FXComposite instances */
%typemap(ruby, out) FXComposite* "$target = FXRbWrapClassInstance(cFX_Composite, $source);";

/* Output typemap for FXCursor instances */
%typemap(ruby, out) FXCursor* "$target = FXRbWrapClassInstance(cFX_Cursor, $source);";

/* Output typemap for FXDQuat instances */
%typemap(ruby, out) FXDQuat* "$target = FXRbWrapClassInstance(cFXDQuat, $source);";

/* Output typemap for FXDragCorner instances */
%typemap(ruby, out) FXDragCorner* "$target = FXRbWrapClassInstance(cFX_DragCorner, $source);";

/* Output typemap for FXFileAssoc instances */
%typemap(ruby, out) FXFileAssoc* "$target = FXRbWrapClassInstance(cFXFileAssoc, $source);";

/* Output typemap for FXFileDict instances */
%typemap(ruby, out) FXFileDict* "$target = FXRbWrapClassInstance(cFX_FileDict, $source);";

/* Output typemap for FXFont instances */
%typemap(ruby, out) FXFont* "$target = FXRbWrapClassInstance(cFX_Font, $source);";

/* Output typemap for FXFontDesc instances */
%typemap(ruby, out) FXFontDesc* "$target = FXRbWrapClassInstance(cFXFontDesc, $source);";

/* Output typemap for FXGLObject instances */
%typemap(ruby, out) FXGLObject* "$target = FXRbWrapClassInstance(cFX_GLObject, $source);";

/* Output typemap for FXGLPoint instances */
%typemap(ruby, out) FXGLPoint* "$target = FXRbWrapClassInstance(cFX_GLPoint, $source);";

/* Output typemap for FXGLShape instances */
%typemap(ruby, out) FXGLShape* "$target = FXRbWrapClassInstance(cFX_GLShape, $source);";

/* Output typemap for FXGLVisual instances */
%typemap(ruby, out) FXGLVisual* "$target = FXRbWrapClassInstance(cFX_GLVisual, $source);";

/* Output typemap for FXHeader instances */
%typemap(ruby, out) FXHeader* "$target = FXRbWrapClassInstance(cFX_Header, $source);";

/* Output typemap for FXHeaderItem instances */
%typemap(ruby, out) FXHeaderItem* "$target = FXRbWrapClassInstance(cFX_HeaderItem, $source);";

/* Output typemap for FXHMat instances */
%typemap(ruby, out) FXHMat* "$target = FXRbWrapClassInstance(cFXHMat, $source);";

/* Output typemap for FXHVec instances */
%typemap(ruby, out) FXHVec* "$target = FXRbWrapClassInstance(cFXHVec, $source);";

/* Output typemap for FXIcon instances */
%typemap(ruby, out) FXIcon* "$target = FXRbWrapClassInstance(cFX_Icon, $source);";

/* Output typemap for FXIconItem instances */
%typemap(ruby, out) FXIconItem* "$target = FXRbWrapClassInstance(cFX_IconItem, $source);";

/* Output typemap for FXImage instances */
%typemap(ruby, out) FXImage* "$target = FXRbWrapClassInstance(cFX_Image, $source);";

/* Output typemap for FXLight instances */
%typemap(ruby, out) FXLight* "$target = FXRbWrapClassInstance(cFXLight, $source);";

/* Output typemap for FXListItem instances */
%typemap(ruby, out) FXListItem* "$target = FXRbWrapClassInstance(cFX_ListItem, $source);";

/* Output typemap for FXMaterial instances */
%typemap(ruby, out) FXMaterial* "$target = FXRbWrapClassInstance(cFXMaterial, $source);";

/* Output typemap for FXMDIChild instances */
%typemap(ruby, out) FXMDIChild* "$target = FXRbWrapClassInstance(cFX_MDIChild, $source);";

/* Output typemap for FXObject instances */
%typemap(ruby, out) FXObject* "$target = FXRbWrapClassInstance(cFX_Object, $source);";

/* Output typemap for FXOption instances */
%typemap(ruby, out) FXOption* "$target = FXRbWrapClassInstance(cFX_Option, $source);";

/* Output typemap for FXPoint instances */
%typemap(ruby, out) FXPoint* "$target = FXRbWrapClassInstance(cFXPoint, $source);";

/* Output typemap for FXPopup instances */
%typemap(ruby, out) FXPopup* "$target = FXRbWrapClassInstance(cFX_Popup, $source);";

/* Output typemap for FXQuat instances */
%typemap(ruby, out) FXQuat* "$target = FXRbWrapClassInstance(cFXQuat, $source);";

/* Output typemap for FXRectangle instances */
%typemap(ruby, out) FXRectangle* "$target = FXRbWrapClassInstance(cFXRectangle, $source);";

/* Output typemap for FXRegion instances */
%typemap(ruby, out) FXRegion* "$target = FXRbWrapClassInstance(cFX_Region, $source);";

/* Output typemap for FXRegistry instances */
%typemap(ruby, out) FXRegistry* "$target = FXRbWrapClassInstance(cFXRegistry, $source);";

/* Output typemap for FXRootWindow instances */
%typemap(ruby, out) FXRootWindow* "$target = FXRbWrapClassInstance(cFX_RootWindow, $source);";

/* Output typemap for FXScrollbar instances */
%typemap(ruby, out) FXScrollbar* "$target = FXRbWrapClassInstance(cFX_Scrollbar, $source);";

/* Output typemap for FXSize instances */
%typemap(ruby, out) FXSize* "$target = FXRbWrapClassInstance(cFXSize, $source);";

/* Output typemap for FXStatusline instances */
%typemap(ruby, out) FXStatusline* "$target = FXRbWrapClassInstance(cFX_Statusline, $source);";

/* Output typemap for FXStream instances */
%typemap(ruby, out) FXStream* "$target = FXRbWrapClassInstance(cFXStream, $source);";

/* Output typemap for FXStringDict instances */
%typemap(ruby, out) FXStringDict* "$target = FXRbWrapClassInstance(cFX_StringDict, $source);";

/* Output typemap for FXTablePos instances */
%typemap(ruby, out) FXTablePos* "$target = FXRbWrapClassInstance(cFXTablePos, $source);";

/* Output typemap for FXTreeItem instances */
%typemap(ruby, out) FXTreeItem* "$target = FXRbWrapClassInstance(cFX_TreeItem, $source);";

/* Output typemap for FXVec instances */
%typemap(ruby, out) FXVec* "$target = FXRbWrapClassInstance(cFXVec, $source);";

/* Output typemap for FXVerticalFrame instances */
%typemap(ruby, out) FXVerticalFrame* "$target = FXRbWrapClassInstance(cFX_VerticalFrame, $source);";

/* Output typemap for FXViewport instances */
%typemap(ruby, out) FXViewport* "$target = FXRbWrapClassInstance(cFXViewport, $source);";

/* Output typemap for FXVisual instances */
%typemap(ruby, out) FXVisual* "$target = FXRbWrapClassInstance(cFX_Visual, $source);";

/* Output typemap for FXWindow instances */
%typemap(ruby, out) FXWindow* "$target = FXRbWrapClassInstance(cFX_Window, $source);";

/* Convert a Ruby array of FXHiliteStyle instances into a C++ array */
%typemap(ruby, in) const FXHiliteStyle* styles {
  Check_Type($source, T_ARRAY);
  $target = new FXHiliteStyle[RARRAY($source)->len];
  for (long i = 0; i < RARRAY($source)->len; i++) {
    FXHiliteStyle* ptr;
    Get_FXHiliteStyle(rb_ary_entry($source, i), ptr);
    $target[i] = *ptr;
  }
}

/* Frees the temporary array allocated in previous typemap */
%typemap(ruby, freearg) const FXHiliteStyle* styles "delete [] $source;";

/* Convert an C++ array of FXHiliteStyle instances into a Ruby array */
%typemap(ruby, out) const FXHiliteStyle* {
  $target = rb_ary_new();
  /*
  for (long i = 0; i < nstyles; i++) {
    rb_ary_store($target, i, Wrap_FXHiliteStyle(cFXHiliteStyle, $source[i], 0));
  }
  */
}
